@@ -195,7 +195,10 @@ void AI::AssignObjectives( void ) {
     last = NULL;
     while ( (u = ClosestUnit( player, obj->pos, U_GROUND|U_AIR|U_SHIP, U_BUSY|U_TRANSPORT, last )) ) {
       if ( u->IsMine() ) u->SetFlags( U_BUSY|U_DONE );
-      else if ( (!u->IsConquer() || (obj->type == AI_OBJ_CONQUER)) &&      // use infantry only
+      else if ( u->Moves() == 0 ) {
+        // stationary units may attack anything that moves...
+        attack_all->AssignUnit( u, 0 );
+      } else if ( (!u->IsConquer() || (obj->type == AI_OBJ_CONQUER)) &&      // use infantry only
                 UnitCanReach( u, obj->pos, AI_ATTENTION_RADIUS ) ) {       // for taking buildings
         const UnitType *type = u->Type();
         if ( ((obj->needed_air > 0) && (type->Firepower(U_AIR) > 0)) ||
