@@ -41,7 +41,7 @@ public:
 
   unsigned char CrystalProduction( void ) const { return b_cprod; }
   unsigned short Crystals( void ) const { return b_crystals; }
-  unsigned short CrystalStore( void ) const { return b_crystalstore; }
+  unsigned short MaxCrystals( void ) const { return b_crystalstore; }
   unsigned short ID( void ) const { return b_id; }
   const char *Name( void ) const { return b_name; }
   signed char NameID( void ) const { return b_name_id; }
