@@ -57,6 +57,9 @@ short UnitContainer::InsertUnit( Unit *unit ) {
     if ( unit->IsTransport() ) {
       Transport *t = static_cast<Transport *>(unit);
 
+      SetCrystals( Crystals() + t->Crystals() );
+      t->SetCrystals( 0 );
+
       while ( Unit *u = t->GetUnit(0) ) {
         t->RemoveUnit( u );
         InsertUnit( u );
@@ -87,7 +90,9 @@ void UnitContainer::RemoveUnit( Unit *unit ) {
         delete n;
 
         unit->UnsetFlags( U_SHELTERED );
-        uc_slots_full -= unit->Weight();
+        // only subtract unit's own weight even for transports
+        // since carried units are removed separately
+        uc_slots_full -= unit->Unit::Weight();
         break;
       }
       n = static_cast<UCNode *>( n->Next() );
@@ -123,6 +128,9 @@ bool UnitContainer::Allow( const Unit *unit ) const {
 
   if ( unit->IsTransport() ) {
     const Transport *t = static_cast<const Transport *>(unit);
+
+    if ( Crystals() + t->Crystals() > MaxCrystals() ) return false;
+
     for ( i = t->UnitCount() - 1; i >= 0; --i )
       if ( !Allow( t->GetUnit(i) ) ) return false;
   }
@@ -248,7 +256,18 @@ bool Transport::Hit( unsigned short damage ) {
 ////////////////////////////////////////////////////////////////////////
 
 bool Transport::Allow( const Unit *unit ) const {
-  if ( unit->Owner() == u_player ) return UnitContainer::Allow( unit );
+  if ( unit->Owner() == u_player ) {
+
+    if ( unit->IsTransport() ) {
+      const Transport *t = static_cast<const Transport *>(unit);
+      if ( uc_slots < uc_slots_full - (Crystals()+9)/10
+                      + unit->Weight()
+                      + (Crystals() + t->Crystals() + 9)/10 )
+        return false;
+    }
+
+    return UnitContainer::Allow( unit );
+  }
   return false;
 }
 
