@@ -139,6 +139,37 @@ bool Event::CheckTrigger( void ) {
         rc = ( b && b->Owner() && (b->Owner()->ID() == e_tdata[1]) );
       }
       break;
+    case ETRIGGER_HAVE_CRYSTALS: {
+      unsigned short crystals = 0;
+      Building *b;
+      if ( e_tdata[2] >= 0 ) {
+        b = mis->GetShop( e_tdata[2] );
+        if ( b && b->Owner() && (b->Owner()->ID() == e_tdata[1]) )
+          crystals = b->Crystals();
+      } else {
+        for ( b = static_cast<Building *>(mis->GetShops().Head()); b;
+              b = static_cast<Building *>(b->Next()) ) {
+          if ( b->Owner() && (b->Owner()->ID() == e_tdata[1]) )
+            crystals += b->Crystals();
+        }
+
+        if ( (e_tdata[2] == -2) &&
+             (((e_tdata[0] < 0) && (crystals < -e_tdata[0])) ||
+              ((e_tdata[0] > 0) && (crystals < e_tdata[0]))) ) {
+          // also check units
+          for ( Unit *u = static_cast<Unit *>(mis->GetUnits().Head()); u;
+                u = static_cast<Unit *>(u->Next()) ) {
+            if ( u->IsTransport() && !u->IsSheltered() &&
+                 u->Owner() && (u->Owner()->ID() == e_tdata[1]) &&
+                 u->IsAlive() ) {
+              crystals += static_cast<Transport *>(u)->Crystals();
+            }
+          }
+        }
+      }
+      rc = ((e_tdata[0] < 0) && (crystals < -e_tdata[0])) ||
+            ((e_tdata[0] > 0) && (crystals >= e_tdata[0]));
+      break; }
     case ETRIGGER_HAVE_UNIT:
       if ( (e_tdata[2] == -1) || (mis->GetTime() >= e_tdata[2]) ) {
         Unit *u = mis->GetUnit( e_tdata[0] );
@@ -149,13 +180,28 @@ bool Event::CheckTrigger( void ) {
       Point loc = mis->GetMap().Index2Hex(e_tdata[1]);
       if ( e_tdata[0] < 0 ) {
         // -1 means any unit, -X is unit of type X - 2
-        for ( Unit *u = static_cast<Unit *>(mis->GetUnits().Head()); u;
-              u = static_cast<Unit *>(u->Next()) ) {    
-          if ( (u->Position() == loc) && u->Owner() &&
-               (u->Owner()->ID() == e_tdata[2]) &&
-               ((e_tdata[0] == -1) || (-e_tdata[0]-2 == u->Type()->ID())) ) {
-            rc = true;
-            break;
+        MapObject *mo = mis->GetMap().GetMapObject( loc );
+        if ( mo && mo->Owner() && (mo->Owner()->ID() == e_tdata[2]) ) {
+          if (e_tdata[0] == -1) {
+            rc = mo->IsUnit() || (static_cast<Building *>(mo)->UnitCount() > 0);
+          } else {
+            unsigned short type = -e_tdata[0] - 2;
+            UnitContainer *c = NULL;
+
+            if ( mo->IsUnit() ) {
+              Unit *u = static_cast<Unit *>(mo);
+              if ( u->Type()->ID() == type ) rc = true;
+              else if ( u->IsTransport() ) c = static_cast<Transport *>(u);
+            } else c = static_cast<Building *>(mo);
+
+            if ( c ) {
+              for ( int i = c->UnitCount() - 1; i >= 0; --i ) {
+                if ( c->GetUnit( i )->Type()->ID() == type ) {
+                  rc = true;
+                  break;
+                }
+              }
+            }
           }
         }
       } else {
@@ -420,6 +466,12 @@ Point Event::GetFocus( void ) const {
       mo = mis->GetShop( e_tdata[0] );
       if ( mo ) p = mo->Position();
       break;
+    case ETRIGGER_HAVE_CRYSTALS:
+      if ( e_tdata[0] >= 0 ) {
+        mo = mis->GetShop( e_tdata[0] );
+        if ( mo ) p = mo->Position();
+      }
+      break;
     case ETRIGGER_HAVE_UNIT:
       mo = mis->GetUnit( e_tdata[0] );
       if ( mo ) p = mo->Position();
