@@ -26,6 +26,7 @@
 #include "game.h"
 #include "path.h"
 #include "unitwindow.h"
+#include "gamewindow.h"
 #include "filewindow.h"
 #include "initwindow.h"
 #include "fileio.h"
@@ -780,7 +781,7 @@ void Game::SetCursor( const Point &cursor ) const {
         if ( target ) {
           Point p = mv->Hex2Pixel( *target );
           mv->DrawTerrain( IMG_CURSOR_HIGHLIGHT, mwin, p.x, p.y, *mwin);
-          mwin->Show( Rect(p.x, p.y, GFX_WIDTH, GFX_HEIGHT) );
+          mwin->Show( Rect(p.x, p.y, mv->TileWidth(), mv->TileHeight()) );
         }
       }
     }
@@ -804,14 +805,37 @@ void Game::SetCursor( const Point &cursor ) const {
 ////////////////////////////////////////////////////////////////////////
 
 Unit *Game::MoveUnit( Unit *u, const Point &dest ) {
-  if ( mission->GetPlayer().IsHuman() &&
-       (shader->GetStep(dest) == -1) ) return u;
-
   if ( u->Position() != dest ) {
     MapView *mv = mwin->GetMapView();
     Map *map = mv->GetMap();
     const Point &pos = u->Position();
 
+    if ( mission->GetPlayer().IsHuman() ) {
+      if ( shader->GetStep(dest) == -1 ) return u;
+
+      // if we're moving a transport out of a shop/transport check
+      // whether the player wants to take other units with him
+      if ( u->IsTransport() && u->IsSheltered() ) {
+        MapObject *pobj = map->GetMapObject( pos );
+        UnitContainer *parent;
+        if ( pobj->IsUnit() ) parent = static_cast<Transport *>(pobj);
+        else parent = static_cast<Building *>(pobj);
+        UnitLoadWindow *ulw = new UnitLoadWindow(
+            *static_cast<Transport *>(u), *parent,
+            *map->GetUnitSet(), *map->GetTerrainSet(), view );
+
+        bool aborted = false;
+        if ( ulw->Opened() ) aborted = (ulw->EventLoop() == GUI_CLOSE);
+        view->CloseWindow( ulw );
+
+        if ( aborted ) {
+          DeselectUnit();
+          ContainerContent( parent );
+          return u;
+        }
+      }
+    }
+
     Path path( map );
     if ( path.Find( u, pos, dest ) == -1 ) {
       cerr << "Internal error: FindPath() failed!" << endl;
@@ -876,7 +900,7 @@ int Game::MoveUnit( Unit *u, Direction dir ) {
 
   if ( mv->Enabled() &&
        (mv->HexVisible(pos) || mv->HexVisible(posnew)) )
-    mwin->MoveHex( u->Image(), mission->GetUnitSet().GetTiles(),
+    mwin->MoveHex( u->Image(), mission->GetUnitSet(),
                    pos, posnew, ANIM_SPEED_UNIT );
 
   u->SetPosition( posnew.x, posnew.y );
@@ -890,7 +914,7 @@ int Game::MoveUnit( Unit *u, Direction dir ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Game::RemoveUnit
 // DESCRIPTION: Remove a unit from the map.
-// PARAMETERS : u  - unit to be removed
+// PARAMETERS : u - unit to be removed
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -1133,6 +1157,15 @@ void Game::Undo( void ) {
   RemoveUnit( u );
   u->Face( undo.GetDirection() );
   u->UnsetFlags( U_DONE|U_MOVED );
+
+  // if the unit is a transport and came out of another
+  // container we also need to reset the carried units' flags
+  if ( u->IsTransport() && map.GetMapObject( undo.GetPosition() ) ) {
+    Transport *t = static_cast<Transport *>(u);
+    for ( int i = 0; i < t->UnitCount(); ++i )
+      t->GetUnit( i )->UnsetFlags( U_DONE|U_MOVED );
+  }
+
   map.SetUnit( u, undo.GetPosition() );
   mwin->GetMapView()->UpdateHex( undo.GetPosition() );
   mwin->Show();
@@ -1329,8 +1362,8 @@ void Game::SelectCommand( const Point &hex ) {
 void Game::ScrollCommand( int key ) {
   MapView *mv = mwin->GetMapView();
   Point pos = mv->Cursor();
-  int sw = mv->Width() * 3 / 4 / GFX_WIDTH;
-  int sh = mv->Height() * 3 / 4 / GFX_HEIGHT;
+  int sw = mv->Width() * 3 / 4 / mv->TileWidth();
+  int sh = mv->Height() * 3 / 4 / mv->TileHeight();
 
   switch ( key ) {
   case SDLK_KP1: pos.x -= sw; pos.y += sh; break;
@@ -1403,7 +1436,7 @@ void Game::ContainerContent( UnitContainer *c ) {
 
 void Game::UnitInfo( Unit *unit ) {
   if ( unit->Owner() == &mission->GetPlayer() )
-    new UnitInfoWindow( unit->Type(), mwin, view );
+    new UnitInfoWindow( unit->Type()->ID(), mission->GetMap(), view );
   else new NoteWindow( unit->Name(), MSG(MSG_ERR_NO_ACCESS), WIN_CLOSE_ESC, view );
 }
 
