@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -63,9 +63,7 @@ int HistEvent::Save( File &file ) const {
 ////////////////////////////////////////////////////////////////////////
 
 short History::Load( File &file, Mission &mission ) {
-  unsigned short num_events;
-
-  num_events = file.Read16();
+  unsigned short num_events = file.Read16();
 
   if ( num_events > 0 ) {
     unsigned short num_units, i;
@@ -81,9 +79,8 @@ short History::Load( File &file, Mission &mission ) {
       Unit *u = mission.LoadUnit( file, true );
       if ( u ) units.AddTail( u );
     }
-    return num_events;
   }
-  return 0;
+  return num_events;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -159,7 +156,7 @@ void History::Replay( MapWindow *mapwin ) {
       unsigned short count = events.CountNodes();
 
       // undo the map tile changes which have been recorded this turn
-     while ( he && (he->type == HIST_TILE_INTERNAL) ) {
+      while ( he && (he->type == HIST_TILE_INTERNAL) ) {
         map->SetHexType( he->data[1], he->data[2], he->data[0] );
         he = static_cast<HistEvent *>( he->Next() );
         --count;
@@ -184,13 +181,15 @@ void History::Replay( MapWindow *mapwin ) {
         ReplayCombatEvent( *he, mapwin );
       else if ( he->type == HIST_MSG )
         ReplayMessageEvent( *he, view );
+      else if ( he->type == HIST_TILE )
+        ReplayTileEvent( *he, mapwin );
+      else if ( he->type == HIST_UNIT )
+        ReplayUnitEvent( *he, mapwin );
 
       else if ( !quick ) {
-        // these events are skipped for quick replays
+        // these events are skipped completely for quick replays
         if ( he->type == HIST_ATTACK )
           ReplayAttackEvent( *he, mapwin );
-        else if ( he->type == HIST_TILE )
-          ReplayTileEvent( *he, mapwin );
       }
 
       if ( !quick ) {
@@ -238,12 +237,16 @@ void History::BeginReplay( List &backup, Map *map ) {
     if ( !u->IsSheltered() ) map->SetUnit( NULL, u->Position() );
   }
 
-  while ( !units.IsEmpty() ) {
-    u = static_cast<Unit *>( units.RemHead() );
-    gam_units.AddTail( u );
-    if ( !u->IsSheltered() ) {
-      map->SetUnit( u, u->Position() );
-      if ( u->IsMine() ) u->SetMoves( MCOST_MIN );
+  Unit *next = NULL;
+  for ( u = static_cast<Unit *>(units.Head()); u; u = next ) {
+    next = static_cast<Unit *>( u->Next() );
+
+    // we use the BUSY flag internally to signal units which do
+    // not yet exist at the time replay is started
+    if ( !u->IsBusy() ) {
+      u->Remove();
+      gam_units.AddTail( u );
+      if ( !u->IsSheltered() ) map->SetUnit( u, u->Position() );
     }
   }
 
@@ -292,6 +295,7 @@ int History::RecordMoveEvent( const Unit &u, Direction dir ) {
   he->type = HIST_MOVE;
   he->data[0] = u.ID();
   he->data[1] = dir;
+  he->data[2] = he->data[3] = -1;
   events.AddTail( he );
   return 0;
 }
@@ -311,6 +315,7 @@ int History::RecordAttackEvent( const Unit &u, const Unit &target ) {
   he->data[0] = u.ID();
   he->data[1] = target.Position().x;
   he->data[2] = target.Position().y;
+  he->data[3] = -1;
 
   events.AddTail( he );
   return 0;
@@ -362,6 +367,7 @@ int History::RecordTileEvent( unsigned short tile, unsigned short old,
   he->data[0] = tile;
   he->data[1] = dx;
   he->data[2] = dy;
+  he->data[3] = -1;
   events.AddTail( he );
 
   // now create the map state initializer event and put it at the
@@ -371,6 +377,7 @@ int History::RecordTileEvent( unsigned short tile, unsigned short old,
   he->data[0] = old;
   he->data[1] = dx;
   he->data[2] = dy;
+  he->data[3] = -1;
   events.AddHead( he );
   return 0;
 }
@@ -381,29 +388,52 @@ int History::RecordTileEvent( unsigned short tile, unsigned short old,
 // PARAMETERS : title - title to use for the message window. If -1,
 //                      used title will be "Message".
 //              msg   - index of the message
+//              pos   - location index of a hex on the map that should
+//                      be shown when displaying the message
 // RETURNS    : 0 on success, -1 on error
 ////////////////////////////////////////////////////////////////////////
 
-int History::RecordMsgEvent( short title, unsigned short msg ) {
+int History::RecordMsgEvent( short title, unsigned short msg, short pos ) {
   HistEvent *he = new HistEvent;
 
   he->type = HIST_MSG;
   he->data[0] = title;
   he->data[1] = msg;
-  he->data[2] = he->data[3] = -1;
+  he->data[2] = pos;
+  he->data[3] = -1;
 
   events.AddTail( he );
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::RecordUnitEvent
+// DESCRIPTION: Record the creation of a new unit.
+// PARAMETERS : unit - newly created unit
+// RETURNS    : 0 on success, -1 on error
+////////////////////////////////////////////////////////////////////////
+
+int History::RecordUnitEvent( const Unit &unit ) {
+  HistEvent *he = new HistEvent;
+
+  he->type = HIST_UNIT;
+  he->data[0] = unit.ID();
+  he->data[1] = he->data[2] = he->data[3] = -1;
+
+  events.AddTail( he );
+
+  Unit *clone = new Unit( unit );
+  clone->SetFlags( U_BUSY|U_DUMMY );
+  units.AddTail( clone );
+  return 0;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : History::ReplayMoveEvent
 // DESCRIPTION: Show the movement of a unit from one hex to another,
 //              neighboring hex.
 // PARAMETERS : event  - move event
 //              mapwin - map window
-//              quick  - for quick replays we only need to update the
-//                       unit positions
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -430,10 +460,7 @@ void History::ReplayMoveEvent( const HistEvent &event, MapWindow *mapwin ) {
       SDL_Delay( delay );
     }
 
-    if ( mv->Enabled() && !mv->HexVisible( u->Position() ) ) {
-      mv->CenterOnHex( u->Position() );
-      mapwin->Show( *mv );
-    }
+    mapwin->DisplayHex( u->Position() );
 
     if ( u->IsSheltered() ) {
       if ( map->GetUnit( u->Position() ) ) u->UnsetFlags( U_SHELTERED );
@@ -494,9 +521,15 @@ void History::ReplayAttackEvent( const HistEvent &event, MapWindow *mapwin ) {
 ////////////////////////////////////////////////////////////////////////
 
 void History::ReplayMessageEvent( const HistEvent &event, View *view ) {
+  MapWindow *mwin = Gam->GetMapWindow();
+  Mission *m = Gam->GetMission();
+
+  if ( mwin->GetMapView()->Enabled() && (event.data[2] >= 0) )
+    mwin->DisplayHex( m->GetMap().Index2Hex( event.data[2] ) );
+
   MessageWindow *msgw = new MessageWindow(
-    (event.data[0] != -1) ? Gam->GetMission()->GetMessage( event.data[0] ) : MSG(MSG_MESSAGE),
-    Gam->GetMission()->GetMessage( event.data[1] ), view );
+    (event.data[0] != -1) ? m->GetMessage( event.data[0] ) : MSG(MSG_MESSAGE),
+    m->GetMessage( event.data[1] ), view );
   msgw->EventLoop();
   view->CloseWindow( msgw );
 }
@@ -512,10 +545,16 @@ void History::ReplayMessageEvent( const HistEvent &event, View *view ) {
 void History::ReplayTileEvent( const HistEvent &event, MapWindow *mapwin ) {
   MapView *mv = mapwin->GetMapView();
   Map *map = mv->GetMap();
+  Point pos( event.data[1], event.data[2] );
 
+  if ( mv->Enabled() ) {
+    const TerrainType *tt = Gam->GetMission()->GetTerrainSet().GetTerrainInfo( event.data[0] );
+    mapwin->DisplayHex( pos );
+    SDL_Delay( delay );
+    mapwin->FadeInTerrain( tt->tt_image, pos );
+    SDL_Delay( delay );
+  }
   map->SetHexType( event.data[1], event.data[2], event.data[0] );
-  mapwin->Show( mv->UpdateHex( Point(event.data[1], event.data[2]) ) );
-  SDL_Delay( delay );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -545,3 +584,58 @@ void History::ReplayCombatEvent( const HistEvent &event, MapWindow *mapwin ) {
   Gam->ResolveBattle( &cmb );
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::ReplayUnitEvent
+// DESCRIPTION: Add a new unit to the map.
+// PARAMETERS : event  - unit event
+//              mapwin - map window
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void History::ReplayUnitEvent( const HistEvent &event, MapWindow *mapwin ) {
+  Mission *mission = Gam->GetMission();
+  MapView *mv = mapwin->GetMapView();
+  Map *map = mv->GetMap();
+
+  for ( Unit *u = static_cast<Unit *>( units.Head() );
+        u; u = static_cast<Unit *>( u->Next() ) ) {
+    if ( u->ID() == event.data[0] ) {
+      u->Remove();
+
+      if ( !u->IsSheltered() ) {
+        if ( mv->Enabled() ) {
+          mapwin->DisplayHex( u->Position() );
+          SDL_Delay( delay );
+          mapwin->FadeInUnit( u->Image(), u->Position() );
+          SDL_Delay( delay );
+        }
+        map->SetUnit( u, u->Position() );
+      }
+      mission->GetUnits().AddTail( u );
+      break;
+    }
+  }
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : History::EraseMoveEvents
+// DESCRIPTION: Erase all recorded movement for a spcific unit.
+// PARAMETERS : u - unit to erase movement for
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void History::EraseMoveEvents( const Unit &u ) {
+  HistEvent *he = static_cast<HistEvent *>( events.Head() ), *tmp;
+
+  while ( he ) {
+    tmp = static_cast<HistEvent *>( he->Next() );
+
+    if ( (he->type == HIST_MOVE) && (he->data[0] == u.ID()) ) {
+      he->Remove();
+      delete he;
+    }
+
+    he = tmp;
+  }
+}
+
