@@ -278,9 +278,9 @@ GUI_Status InitWindow::WidgetActivated( Widget *button, Window *win ) {
         switch (mtypewidget->GetValue() ) {
         case MAPTYPE_SINGLE:
           if ( CFOptions.IsAI() ) {
-            string pnames;
-            pnames.assign( m->GetPlayer(PLAYER_ONE).Name() );
-            pnames += '|';
+            string pnames( "[_1] " );
+            pnames += m->GetPlayer(PLAYER_ONE).Name();
+            pnames += "|[_2] ";
             pnames += m->GetPlayer(PLAYER_TWO).Name();
 
             DialogWindow *pwin = new DialogWindow( MSG(MSG_PLAYER_SELECTION),
@@ -863,8 +863,7 @@ LocaleOptionsWindow::LocaleOptionsWindow( Game *game, View *view ) :
   // create widgets
   const Rect &b = GetBounds();
 
-  locwidget = new TextListWidget( 0, b.x + 5, b.y + 5, b.w - 10, b.h - 10,
-                  &locales, current,
+  locwidget = new TextListWidget( 0, b.x, b.y, b.w, b.h, &locales, current,
                   WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY|WIDGET_ALIGN_CENTER,
                   NULL, this );
 
