@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -43,10 +43,12 @@ public:
 
   void Close( void ) { Window::Close(); view->CloseWindow(title); }
   GUI_Status WidgetActivated( Widget *button, Window *win );
+  void VideoModeChange( void );
 
 private:
   void CompleteFilesList( TLWList &list );
-  Mission *LoadMission( const char *filename ) const;
+  Mission *LoadMission( const char *filename, bool full = true ) const;
+  void Rebuild( void );
 
   CycleWidget *gtypewidget;
   CycleWidget *mtypewidget;
