@@ -1,5 +1,5 @@
 // Crimson Fields -- a hex-based game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ int main( int argc, char **argv ) {
       Mission *m = Gam->GetMission();
       if ( !intro ) {
         // default is to play single-player single-map
-        if ( !m->GetFlags() & GI_SAVEFILE )
+        if ( !(m->GetFlags() & GI_SAVEFILE) )
           m->GetPlayer(PLAYER_ONE).SetType( HUMAN );
         Gam->InitWindows();
         Gam->StartTurn();
@@ -362,7 +362,8 @@ void load_settings( GUIOptions &opts ) {
   if ( file.is_open() ) {
     string buf;
     const char *val, *linebuf;
-    unsigned int pos, line = 0;
+    unsigned int line = 0;
+    size_t pos;
 
     while (!file.eof()) {
       getline(file, buf);
