@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@ public:
   unsigned short TerrainTypes( const Point &hex ) const { return HexType(hex)->tt_type; }
   bool IsWater( const Point &hex ) const
         { return (TerrainTypes(hex) & (TT_WATER|TT_WATER_SHALLOW|TT_WATER_DEEP)) != 0; }
+  bool IsShop( const Point &hex ) const { return (TerrainTypes(hex) & TT_ENTRANCE) != 0; }
 
   unsigned long HexColor( unsigned short xy ) const;
   void SetHexType( short x, short y, short type )
@@ -73,6 +74,7 @@ public:
 
   short GetNeighbors( const Point &hex, Point *parray ) const;
   int Hex2Index( const Point &hex ) const { return hex.y * m_w + hex.x; }
+  Point Index2Hex( int index ) const;
   int Dir2Hex( const Point &hex, Direction dir, Point &dest ) const;
   bool Contains( const Point &hex ) const;
 
