@@ -38,13 +38,7 @@
 
 MapWindow::MapWindow( short x, short y, unsigned short w, unsigned short h,
                       unsigned short flags, View *view ) :
-    Window( x, y, w, h, flags, view ),
-    speed_icon( view->GetSystemIcons(), 0, 0, ICON_WIDTH, ICON_HEIGHT ),
-    def_icon( view->GetSystemIcons(), 32, 0, ICON_WIDTH, ICON_HEIGHT ),
-    ground_icon( view->GetSystemIcons(), 64, 0, ICON_WIDTH, ICON_HEIGHT ),
-    air_icon( view->GetSystemIcons(), 96, 0, ICON_WIDTH, ICON_HEIGHT ),
-    ship_icon( view->GetSystemIcons(), 128, 0, ICON_WIDTH, ICON_HEIGHT )
-  {
+           Window( x, y, w, h, flags, view ) {
   panel = new Panel( this, view );
   mview = new MapView( this, *this,
               MV_AUTOSCROLL|MV_DISABLE|MV_DISABLE_CURSOR|MV_DISABLE_FOG );
@@ -91,15 +85,15 @@ void MapWindow::DrawBack( short x, short y, unsigned short w, unsigned short h )
 // NAME       : MapWindow::MoveHex
 // DESCRIPTION: Smoothly move a hex image (usually a unit or cursor)
 //              from one hex to another (adjacent) one.
-// PARAMETERS : img     - map tile identifier for the hex image
-//              surface - surface containing the image
-//              hex1    - source hex position
-//              hex2    - destination hex position
-//              speed   - total time (ms) for the animation
+// PARAMETERS : img   - map tile identifier for the hex image
+//              tiles - tile set containing the image
+//              hex1  - source hex position
+//              hex2  - destination hex position
+//              speed - total time (ms) for the animation
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void MapWindow::MoveHex( unsigned short img, Surface *surface,
+void MapWindow::MoveHex( unsigned short img, const TileSet &tiles,
                 const Point &hex1, const Point &hex2, unsigned short speed ) {
   // allocate a new surface where we can store the "background" graphics
   Point psrc = mview->Hex2Pixel( hex1 );
@@ -108,8 +102,8 @@ void MapWindow::MoveHex( unsigned short img, Surface *surface,
   Rect bg;
   bg.x = MIN( psrc.x, pdst.x );
   bg.y = MIN( psrc.y, pdst.y );
-  bg.w = GFX_WIDTH + ABS(psrc.x-pdst.x);
-  bg.h = GFX_HEIGHT + ABS(psrc.y-pdst.y);
+  bg.w = tiles.TileWidth() + ABS(psrc.x-pdst.x);
+  bg.h = tiles.TileHeight() + ABS(psrc.y-pdst.y);
   bg.Clip( *this );
 
   Surface *bgs = new Surface;
@@ -127,8 +121,8 @@ void MapWindow::MoveHex( unsigned short img, Surface *surface,
       // are done in one line
       cx = (short)((pdst.x - psrc.x) * ticks);
       cy = (short)((pdst.y - psrc.y) * ticks);
-      mview->DrawHex( img, surface, this,
-                      psrc.x + cx / speed, psrc.y + cy / speed, bg );
+      tiles.DrawTile( img, this,
+            psrc.x + cx / speed, psrc.y + cy / speed, bg );
 
       Show( bg );
       bgs->Blit( this, Rect( 0, 0, bg.w, bg.h ), bg.x, bg.y );
@@ -153,12 +147,12 @@ void MapWindow::MoveHex( unsigned short img, Surface *surface,
 void MapWindow::FadeInHex( unsigned short img, const Point &hex, bool unit ) {
   unsigned long firstticks, oldticks, ticks;
   Point pos = mview->Hex2Pixel( hex );
-  const Rect dest( pos.x, pos.y, GFX_WIDTH, GFX_HEIGHT );
+  const Rect dest( pos.x, pos.y, mview->TileWidth(), mview->TileHeight() );
   const unsigned int speed = 4 * ANIM_SPEED_UNIT;
   const unsigned short terrain = mview->GetMap()->HexImage( hex );
 
   Surface tile;
-  tile.Create( GFX_WIDTH, GFX_HEIGHT, DISPLAY_BPP, SDL_HWSURFACE );
+  tile.Create( mview->TileWidth(), mview->TileHeight(), DISPLAY_BPP, SDL_HWSURFACE );
   tile.SetAlpha( SDL_ALPHA_TRANSPARENT, SDL_SRCALPHA );
   tile.SetColorKey( Color(CF_COLOR_WHITE) );
   tile.DisplayFormat();
@@ -213,10 +207,10 @@ void MapWindow::BoxAvoidHexes( Rect &box, const Point &hex1, const Point &hex2 )
   Point p2 = mview->Hex2Pixel( hex2 );
 
   for ( int i = 0; i < 4; ++i ) {
-    if ( (((defs[i].x > p1.x + GFX_WIDTH) || (defs[i].x + box.w <= p1.x)) ||
-         ((defs[i].y > p1.y + GFX_HEIGHT) || (defs[i].y + box.h <= p1.y))) &&
-         (((defs[i].x > p2.x + GFX_WIDTH) || (defs[i].x + box.w <= p2.x)) ||
-         ((defs[i].y > p2.y + GFX_HEIGHT) || (defs[i].y + box.h <= p2.y))) ) {
+    if ( (((defs[i].x > p1.x + mview->TileWidth()) || (defs[i].x + box.w <= p1.x)) ||
+         ((defs[i].y > p1.y + mview->TileHeight()) || (defs[i].y + box.h <= p1.y))) &&
+         (((defs[i].x > p2.x + mview->TileWidth()) || (defs[i].x + box.w <= p2.x)) ||
+         ((defs[i].y > p2.y + mview->TileHeight()) || (defs[i].y + box.h <= p2.y))) ) {
       box.x = defs[i].x;
       box.y = defs[i].y;
       return;
@@ -256,114 +250,12 @@ Point MapWindow::MoveCursor( Direction dir ) {
   mview->DisableCursor();
   mview->UpdateHex( cursor );
   MoveHex( mview->GetCursorImage(),
-           mview->GetMap()->GetTerrainSet()->GetTiles(),
+           *mview->GetMap()->GetTerrainSet(),
            cursor, dest, ANIM_SPEED_CURSOR );
   mview->EnableCursor();
   return dest;
 }
 
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : MapWindow::DrawUnitInfo
-// DESCRIPTION: Display unit type information.
-// PARAMETERS : type - unit type (may be NULL)
-//              dest - destination surface
-//              rect - rectangle in which to display
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void MapWindow::DrawUnitInfo( const UnitType *type, Surface *dest,
-                              const Rect &rect ) {
-  char buf[12];
-  short xpos, ypos, xspacing, yspacing, terbreak;
-  const TerrainSet *tset = mview->GetMap()->GetTerrainSet();
-
-  dest->DrawBox( rect, BOX_CARVED );
-  if ( type == NULL ) return;
-
-  yspacing = rect.h - GFX_HEIGHT - (ICON_HEIGHT + sfont->Width()) * 2;
-  if ( rect.w >= rect.h ) {        // draw all terrain on the same level
-    terbreak = 7;
-  } else {                         // draw two terrain levels
-    terbreak = 4;
-    yspacing -= GFX_HEIGHT;
-  }
-  yspacing /= 4;
-
-  sfont->Write( type->Name(), dest,
-                rect.x + (rect.w - sfont->TextWidth(type->Name())) / 2,
-                rect.y + yspacing, rect );
-
-  // show unit specs
-  xspacing = (rect.w - 2 * ICON_WIDTH - 3 * sfont->Width()) / 3;
-  xpos = rect.x + xspacing;
-  ypos = rect.y + sfont->Height() + 2 * yspacing;
-  itoa( type->Speed(), buf );
-  sfont->Write( buf, dest, xpos + ICON_WIDTH + 4,
-                ypos + (ICON_HEIGHT - sfont->Height())/2 );
-  speed_icon.Draw( dest, xpos, ypos );
-
-  xpos += ICON_WIDTH + xspacing;
-  itoa( type->Armour(), buf );
-  sfont->Write( buf, dest, xpos + ICON_WIDTH + 4,
-                ypos + (ICON_HEIGHT - sfont->Height())/2 );
-  def_icon.Draw( dest, xpos, ypos );
-
-  // show unit combat strength
-  xspacing = (rect.w - 3 * ICON_WIDTH) / 4;
-  xpos = rect.x + xspacing;
-  ypos += ICON_HEIGHT - 5;
-  ground_icon.Draw( dest, xpos, ypos );
-  if ( type->Firepower(U_GROUND) == 0 ) strcpy( buf, "-" );
-  else if ( (type->MinFOF(U_GROUND) == type->MaxFOF(U_GROUND))
-         && (type->MaxFOF(U_GROUND) <= 1) ) itoa( type->Firepower(U_GROUND), buf );
-  else sprintf( buf, "%2d (%d-%d)", type->Firepower(U_GROUND),
-                type->MinFOF(U_GROUND), type->MaxFOF(U_GROUND) );
-  sfont->Write( buf, dest, xpos + (ICON_WIDTH - sfont->TextWidth(buf))/2,
-                           ypos + ICON_HEIGHT );
-
-  xpos += ICON_WIDTH + xspacing;
-  air_icon.Draw( dest, xpos, ypos );
-  if ( type->Firepower(U_AIR) == 0 ) strcpy( buf, "-" );
-  else if ( (type->MinFOF(U_AIR) == type->MaxFOF(U_AIR))
-         && (type->MaxFOF(U_AIR) <= 1) ) itoa( type->Firepower(U_AIR), buf );
-  else sprintf( buf, "%2d (%d-%d)", type->Firepower(U_AIR),
-                type->MinFOF(U_AIR), type->MaxFOF(U_AIR) );
-  sfont->Write( buf, dest, xpos + (ICON_WIDTH - sfont->TextWidth(buf))/2,
-                           ypos + ICON_HEIGHT );
-
-  xpos += ICON_WIDTH + xspacing;
-  ship_icon.Draw( dest, xpos, ypos );
-  if ( type->Firepower(U_SHIP) == 0 ) strcpy( buf, "-" );
-  else if ( (type->MinFOF(U_SHIP) == type->MaxFOF(U_SHIP))
-         && (type->MaxFOF(U_SHIP) <= 1) ) itoa( type->Firepower(U_SHIP), buf );
-  else sprintf( buf, "%2d (%d-%d)", type->Firepower(U_SHIP),
-                type->MinFOF(U_SHIP), type->MaxFOF(U_SHIP) );
-  sfont->Write( buf, dest, xpos + (ICON_WIDTH - sfont->TextWidth(buf))/2,
-                           ypos + ICON_HEIGHT );
-
-
-  ypos += ICON_HEIGHT + sfont->Height() + yspacing;
-  // show terrain access
-  Rect terborder( rect.x + 10, ypos, rect.w - 20, rect.h - ypos );
-  dest->FillRect( terborder.x, terborder.y - yspacing/2, terborder.w, 1, Color(CF_COLOR_SHADOW) );
-  dest->FillRect( terborder.x, terborder.y + 1 - yspacing/2, terborder.w, 1, Color(CF_COLOR_HIGHLIGHT) );
-
-  xpos = terborder.x + (terborder.w - GFX_WIDTH * terbreak) / 2;
-  ypos = terborder.y;
-  for ( int i = 0; i < 7; ++i ) {
-    if ( i == terbreak ) {
-      xpos = terborder.x + (terborder.w - GFX_WIDTH * (7 - terbreak)) / 2;
-      ypos += GFX_HEIGHT;
-    }
-
-    const TerrainType *tt = tset->GetTerrainInfo( tset->GetTerrainClassID(i) );
-    mview->DrawTerrain( tt->tt_image, dest, xpos, ypos, rect );
-    if ( !(type->Terrain() & tt->tt_type) ) mview->DrawFog( dest, xpos, ypos, rect );
-    xpos += GFX_WIDTH;
-  }
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapWindow::VideoModeChange
 // DESCRIPTION: This method is called by the view whenever the video
@@ -473,12 +365,12 @@ void Panel::Update( MapObject *obj ) {
 
     MapView *mv = mapwin->GetMapView();
     Point pcursor = mv->Hex2Pixel( mv->Cursor() );
-    if ( pcursor.x <= w + GFX_WIDTH ) {
+    if ( pcursor.x <= w + mv->TileWidth() ) {
       if ( y == 0 ) {
-        if ( pcursor.y <= h + GFX_HEIGHT ) {
+        if ( pcursor.y <= h + mv->TileHeight() ) {
           y = mapwin->Height() - h;
         }
-      } else if ( pcursor.y > y - GFX_HEIGHT ) y = 0;
+      } else if ( pcursor.y > y - mv->TileHeight() ) y = 0;
     }
     SetSize( w, h );   // allocate surface
     Draw();
@@ -689,12 +581,13 @@ CombatWindow::CombatWindow( Combat *combat, MapWindow *mapwin, View *view ) :
     def( combat->GetDefender() ), apos( att->Position() ), dpos( def->Position() ),
     startgroup1( att->GroupSize() ), startgroup2( def->GroupSize() ) {
 
+  MapView *mv = mapwin->GetMapView();
+
   // calculate window dimensions
   short wwidth = (MAX( sfont->TextWidth( att->Name() ),
                        sfont->TextWidth( def->Name() ) ) + XP_ICON_WIDTH + 25) * 2;
-  wwidth = MAX( wwidth, GFX_WIDTH * 6 + 50 );
-  Rect win( 0, 0, wwidth, GFX_HEIGHT * 3 + sfont->Height() * 2 + 50 );
-  MapView *mv = mapwin->GetMapView();
+  wwidth = MAX( wwidth, mv->TileWidth() * 6 + 50 );
+  Rect win( 0, 0, wwidth, mv->TileHeight() * 3 + sfont->Height() * 2 + 50 );
 
   // place window on map window
   mv->CenterOnHex( Point( (apos.x + dpos.x) / 2, (apos.y + dpos.y) / 2 ) );
@@ -704,11 +597,11 @@ CombatWindow::CombatWindow( Combat *combat, MapWindow *mapwin, View *view ) :
   // highlight the fighting units
   Rect clip;
   Point punit = mv->Hex2Pixel( apos );
-  clip = Rect( punit.x, punit.y, GFX_WIDTH, GFX_HEIGHT );
+  clip = Rect( punit.x, punit.y, mv->TileWidth(), mv->TileHeight() );
   clip.Clip( *mv );
   mv->DrawTerrain( IMG_CURSOR_HIGHLIGHT, mapwin, punit.x, punit.y, clip );
   punit = mv->Hex2Pixel( dpos );
-  clip = Rect( punit.x, punit.y, GFX_WIDTH, GFX_HEIGHT );
+  clip = Rect( punit.x, punit.y, mv->TileWidth(), mv->TileHeight() );
   clip.Clip( *mv );
   mv->DrawTerrain( IMG_CURSOR_HIGHLIGHT, mapwin, punit.x, punit.y, clip );
 
@@ -718,21 +611,21 @@ CombatWindow::CombatWindow( Combat *combat, MapWindow *mapwin, View *view ) :
   msgbar1 = Rect( 4, h - sfont->Height() - 14 - brect.h, (w - 16) / 2, sfont->Height() + 6 );
   msgbar2 = Rect( msgbar1.x + msgbar1.w + 8, msgbar1.y, msgbar1.w, msgbar1.h );
 
-  att_anchor = Rect( w/4 - GFX_WIDTH/2, 10 + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
-  def_anchor = Rect( (w * 3)/4 - GFX_WIDTH/2, att_anchor.y, GFX_WIDTH, GFX_HEIGHT );
+  att_anchor = Rect( w/4 - mv->TileWidth()/2, 10 + mv->TileHeight(), mv->TileWidth(), mv->TileHeight() );
+  def_anchor = Rect( (w * 3)/4 - mv->TileWidth()/2, att_anchor.y, mv->TileWidth(), mv->TileHeight() );
 
-  clock[0][0] = Rect( att_anchor.x, att_anchor.y - GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
-  clock[0][1] = Rect( att_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, att_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[0][2] = Rect( att_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, att_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[0][3] = Rect( att_anchor.x, att_anchor.y + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
-  clock[0][4] = Rect( att_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, att_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[0][5] = Rect( att_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, att_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][0] = Rect( def_anchor.x, def_anchor.y - GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][1] = Rect( def_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, def_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][2] = Rect( def_anchor.x + GFX_WIDTH - GFX_OVERLAP_X, def_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][3] = Rect( def_anchor.x, def_anchor.y + GFX_HEIGHT, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][4] = Rect( def_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, def_anchor.y + GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
-  clock[1][5] = Rect( def_anchor.x - GFX_WIDTH + GFX_OVERLAP_X, def_anchor.y - GFX_OVERLAP_Y, GFX_WIDTH, GFX_HEIGHT );
+  clock[0][0] = Rect( att_anchor.x, att_anchor.y - mv->TileHeight(), mv->TileWidth(), mv->TileHeight() );
+  clock[0][1] = Rect( att_anchor.x + mv->TileWidth() - mv->TileShiftX(), att_anchor.y - mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[0][2] = Rect( att_anchor.x + mv->TileWidth() - mv->TileShiftX(), att_anchor.y + mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[0][3] = Rect( att_anchor.x, att_anchor.y + mv->TileHeight(), mv->TileWidth(), mv->TileHeight() );
+  clock[0][4] = Rect( att_anchor.x - mv->TileWidth() + mv->TileShiftX(), att_anchor.y + mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[0][5] = Rect( att_anchor.x - mv->TileWidth() + mv->TileShiftX(), att_anchor.y - mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][0] = Rect( def_anchor.x, def_anchor.y - mv->TileHeight(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][1] = Rect( def_anchor.x + mv->TileWidth() - mv->TileShiftX(), def_anchor.y - mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][2] = Rect( def_anchor.x + mv->TileWidth() - mv->TileShiftX(), def_anchor.y + mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][3] = Rect( def_anchor.x, def_anchor.y + mv->TileHeight(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][4] = Rect( def_anchor.x - mv->TileWidth() + mv->TileShiftX(), def_anchor.y + mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
+  clock[1][5] = Rect( def_anchor.x - mv->TileWidth() + mv->TileShiftX(), def_anchor.y - mv->TileShiftY(), mv->TileWidth(), mv->TileHeight() );
 
   button = new ButtonWidget( GUI_CLOSE, brect.x, brect.y, brect.w, brect.h, WIDGET_DEFAULT, MSG(MSG_B_OK), this );
 
