@@ -67,7 +67,7 @@ public:
   MapView *GetMapView( void ) const { return mview; }
   Panel *GetPanel( void ) const { return panel; }
 
-  void MoveHex( unsigned short img, Surface *surface, const Point &hex1,
+  void MoveHex( unsigned short img, const TileSet &tiles, const Point &hex1,
                 const Point &hex2, unsigned short speed );
   Point MoveCursor( Direction dir );
   void FadeInUnit( unsigned short img, const Point &hex )
@@ -77,18 +77,12 @@ public:
 
   void DisplayHex( const Point &hex );
   void BoxAvoidHexes( Rect &rect, const Point &hex1, const Point &hex2 ) const;
-  void DrawUnitInfo( const UnitType *type, Surface *dest, const Rect &rect );
 
 private:
   void FadeInHex( unsigned short img, const Point &hex, bool unit );
 
   MapView *mview;
   Panel *panel;
-  Image speed_icon;
-  Image def_icon;
-  Image ground_icon;
-  Image air_icon;
-  Image ship_icon;
 };
 
 
