@@ -90,8 +90,7 @@ extern Language Lang;
 #define MSG_ENTER_PASSWORD	(MSG_GAME_SAVED_PBEM+1)
 #define MSG_CHOOSE_PASSWORD	(MSG_ENTER_PASSWORD+1)
 #define MSG_CONFIRM_PASSWORD	(MSG_CHOOSE_PASSWORD+1)
-#define MSG_TRANSFER		(MSG_CONFIRM_PASSWORD+1)
-#define MSG_TRANSFER_UNITS	(MSG_TRANSFER+1)
+#define MSG_TRANSFER_UNITS	(MSG_CONFIRM_PASSWORD+1)
 #define MSG_ASK_SIDE		(MSG_TRANSFER_UNITS+1)
 #define MSG_ASK_REPAIR		(MSG_ASK_SIDE+1)
 #define MSG_ASK_OVERWRITE	(MSG_ASK_REPAIR+1)
@@ -115,9 +114,8 @@ extern Language Lang;
 #define MSG_ERR_SWEEPER_FULL	(MSG_ERR_NO_MAP+1)
 #define MSG_ERR_NO_CRYSTALS	(MSG_ERR_SWEEPER_FULL+1)
 #define MSG_ERR_NO_PRODUCTION	(MSG_ERR_NO_CRYSTALS+1)
-#define MSG_ERR_NO_TRANSPORTER	(MSG_ERR_NO_PRODUCTION+1)
 
-#define CF_MSGS			(MSG_ERR_NO_TRANSPORTER+1)
+#define CF_MSGS			(MSG_ERR_NO_PRODUCTION+1)
 
 #endif   /* _INCLUDE_MSGS_H */
 
