@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,6 +20,10 @@
 // path.cpp -- pathfinding functions
 ////////////////////////////////////////////////////////////////////////
 
+#include <vector>
+#include <algorithm>
+using namespace std;
+
 #include "path.h"
 
 ////////////////////////////////////////////////////////////////////////
@@ -39,7 +43,6 @@ BasicPath::BasicPath( Map *map, signed char *buffer ) {
   buf_private = false;
 
   SetBuffer( buffer );
-  Clear();
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -75,12 +78,11 @@ void BasicPath::Clear( void ) {
 void BasicPath::SetBuffer( signed char *buffer ) {
   if ( buf_private ) delete [] path;
 
-  if ( !buffer ) {
+  buf_private = !buffer;
+  if ( buf_private ) {
     path = new signed char [map->Width() * map->Height()];
-    buf_private = true;
   } else {
     path = buffer;
-    buf_private = false;
   }
 }
 
@@ -92,8 +94,101 @@ void BasicPath::SetBuffer( signed char *buffer ) {
 //              start - hex to start from (can be different from the
 //                      unit's current position)
 //              end   - destination hex
+// RETURNS    : approximate number of turns to reach the destination
+//              hex, or -1 if no valid path was found
+////////////////////////////////////////////////////////////////////////
+
+short BasicPath::Find( const Unit *u, const Point &start, const Point &end ) {
+  vector<PathNode> openlist;
+  greater<PathNode> comp;
+  this->start = start;
+  this->end = end;
+
+  // array to store node cost in so that we don't have to look it
+  // up in the open list
+  unsigned short mapsize = map->Width() * map->Height();
+  short *nodeval = new short [mapsize];
+  for ( int i = 0; i < mapsize; ++i ) {
+    nodeval[i] = -1;
+    path[i] = -1;
+  }
+
+  // to begin, insert the starting node in openlist
+  PathNode pnode;
+
+  // starting from the destination would make it a bit easier to record the actual path
+  // but this way we can support going towards the target until we have reached a certain
+  // distance which is rather useful for AI routines...
+  pnode.pos = start;
+  pnode.eta = ETA( start );
+  pnode.cost = 0;
+  pnode.switched = false;
+  openlist.push_back( pnode );
+
+  while( !openlist.empty() ) {
+    pnode = openlist.front();
+    pop_heap( openlist.begin(), openlist.end(), comp );
+    openlist.pop_back();
+
+    // check for destination
+    if ( StopSearch( pnode ) ) break;
+
+    // not there yet, so let's look at the hexes around it
+    for ( short dir = NORTH; dir <= NORTHWEST; ++dir ) {
+      Point p;
+
+      // get the hex in that direction
+      if ( !map->Dir2Hex( pnode.pos, (Direction)dir, p ) ) {
+        PathNode pnode2;
+        pnode2.pos = p;
+        if ( AddNode( u, pnode, pnode2 ) ) {
+          unsigned short index = map->Hex2Index( p );
+          if ( path[index] == -1 ) {
+            // node has not yet been visited; queue it
+            path[index] = ReverseDir((Direction)dir);
+            nodeval[index] = pnode2.cost;
+            openlist.push_back( pnode2 );
+            push_heap( openlist.begin(), openlist.end(), comp );
+          } else if ( nodeval[index] > pnode2.cost ) {
+            // node has been visited before; look for it in the
+            // list of open nodes and replace it if the cost of
+            // the new one is smaller
+            for ( vector<PathNode>::iterator it = openlist.begin();
+                  it != openlist.end(); ++it ) {
+              if ( (*it).pos == pnode2.pos ) {
+                path[index] = ReverseDir((Direction)dir);
+                nodeval[index] = pnode2.cost;
+                (*it).cost = pnode2.cost;
+                (*it).switched = pnode2.switched;
+                push_heap( openlist.begin(), it+1, comp );
+                break;
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+  delete [] nodeval;
+
+  // check if we reached our destination
+  return FinalizePath( u, pnode );
+}
+
+// PATH - a Path is the object used for normal unit pathfinding,
+//        trying to move a unit from A to B
+//
+////////////////////////////////////////////////////////////////////////
+// NAME       : Path::Find
+// DESCRIPTION: Search a path on the map.
+// PARAMETERS : u     - unit to search a path for; this may be NULL
+//                      only in some special cases (see subclasses)
+//              start - hex to start from (can be different from the
+//                      unit's current position)
+//              end   - destination hex
 //              qual  - path quality/speed trade-off (1 is best, 10 is
-//                      worst but fastest); Default PATH_BEST.
+//                      worst but fastest). Default PATH_BEST.
 //              off   - sometimes you don't want to reach the target
 //                      hex proper but only get close to it. This is
 //                      the maximum distance to the destination for a
@@ -102,69 +197,27 @@ void BasicPath::SetBuffer( signed char *buffer ) {
 //              hex, or -1 if no valid path was found
 ////////////////////////////////////////////////////////////////////////
 
-short BasicPath::Find( const Unit *u, const Point &start, const Point &end,
-                       unsigned char qual, unsigned char off ) {
-  PathNode *pnode;
-  List openlist, donelist;
-  this->start = start;
-  this->end = end;
-
-  // to begin, insert the starting node in openlist
-  pnode = new PathNode;
-  if ( !pnode ) return -1;
-
-  // starting from the destination would make it a bit easier to record the actual path
-  // but this way we can support going towards the target until we have reached a certain
-  // distance which is rather useful for AI routines...
-  pnode->pos = start;
-  pnode->eta = Distance( start, end ) * qual;
-  pnode->cost = 0;
-  pnode->switched = false;
-  openlist.AddHead( pnode );
-
-  while( !openlist.IsEmpty() ) {
-    pnode = GetBestNode( openlist );
-    pnode->Remove();
-    donelist.AddHead( pnode );
-
-    // check for destination
-    if ( StopSearch( pnode, off ) ) break;
-
-    // not there yet, so let's look at the hexes around it
-    for ( short dir = NORTH; dir <= NORTHWEST; ++dir ) {
-      Point p;
-
-      // get the hex in that direction
-      if ( !map->Dir2Hex( pnode->pos, (Direction)dir, p ) ) {
-        PathNode *pnode2 = GetCost( u, pnode, p );
-        if ( pnode2 ) {
-          pnode2->eta = Distance( p, end ) * qual;
-          MarkBuffer( pnode2, (Direction)dir, openlist );
-        }
-      }
-    }
-  }
-
-  // check if we reached our destination
-  pnode = static_cast<PathNode *>( donelist.Head() );
-  return (pnode ? RecordPath( u, pnode, off ) : -1 );
+short Path::Find( const Unit *u, const Point &start, const Point &end,
+                  unsigned char qual, unsigned char off ) {
+  quality = qual;
+  deviation = off;
+  return BasicPath::Find( u, start, end );
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : BasicPath::GetStep
-// DESCRIPTION: Get the path buffer content at a hex position. Usually,
-//              it will contain the direction of the next step on the
-//              path.
-// PARAMETERS : current - hex position to look at
-// RETURNS    : content of path buffer at the position of 'current'
+// NAME       : Path::ETA
+// DESCRIPTION: Estimate the cost to the destination hex. This is used
+//              as an heuristic for the path finding algorithm.
+// PARAMETERS : current - current location
+// RETURNS    : estimated cost to destination
 ////////////////////////////////////////////////////////////////////////
 
-signed char BasicPath::GetStep( const Point &current ) const {
-  return path[map->Hex2Index(current)];
+inline unsigned short Path::ETA( const Point &p ) const {
+  return Distance( p, end ) * quality;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : BasicPath::StepsToDest
+// NAME       : Path::StepsToDest
 // DESCRIPTION: Get the number of steps on the path from a given hex to
 //              the destination.
 // PARAMETERS : pos - hex position to start from
@@ -172,7 +225,7 @@ signed char BasicPath::GetStep( const Point &current ) const {
 //              no path available or already there
 ////////////////////////////////////////////////////////////////////////
 
-unsigned short BasicPath::StepsToDest( const Point &pos ) const {
+unsigned short Path::StepsToDest( const Point &pos ) const {
   Point p( pos );
   short index = map->Hex2Index( p );
   Direction dir = (Direction)path[index];
@@ -187,130 +240,65 @@ unsigned short BasicPath::StepsToDest( const Point &pos ) const {
   return steps;
 }
 
-
-// PATH - a Path is the object used for normal unit pathfinding,
-//        trying to move a unit from A to B
-//
-//
-////////////////////////////////////////////////////////////////////////
-// NAME       : Path::GetBestNode
-// DESCRIPTION: The best node for standard pathfinding is the one with
-//              the smallest sum of cost and eta.
-// PARAMETERS : open - list of open nodes
-// RETURNS    : best node
-////////////////////////////////////////////////////////////////////////
-
-PathNode *Path::GetBestNode( const List &open ) const {
-  PathNode *p1, *p2;
-
-  p1 = static_cast<PathNode *>( open.Head() );
-  p2 = static_cast<PathNode *>( p1->Next() );
-  while ( p2 ) {
-    if ( (p2->cost + p2->eta) < (p1->cost + p1->eta) ) p1 = p2;
-    p2 = static_cast<PathNode *>( p2->Next() );
-  }
-  return p1;
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Path::StopSearch
 // DESCRIPTION: For the normal pathfinder, the search ends when the
 //              destination is reached (or we're sufficiently close).
 // PARAMETERS : next - path node to be checked next if search is not
 //                     cancelled
-//              off  - minimum distance to end to abort
 // RETURNS    : TRUE if search aborted, FALSE otherwise
 ////////////////////////////////////////////////////////////////////////
 
-bool Path::StopSearch( const PathNode *next, unsigned char off ) const {
-  return Distance( next->pos, end ) <= off;
+inline bool Path::StopSearch( const PathNode &next ) const {
+  return Distance( next.pos, end ) <= deviation;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : Path::GetCost
+// NAME       : Path::AddNode
 // DESCRIPTION: Calculate the cost for the unit to move from one hex to
-//              another.
-// PARAMETERS : u       - unit
-//              pn_from - path node for source hex
-//              to      - destination hex (adjacent to from)
-// RETURNS    : new PathNode or NULL if step is not allowed
+//              another and complete the respective PathNode.
+// PARAMETERS : u    - unit
+//              from - path node for source hex
+//              to   - destination path node (adjacent to from). Only
+//                     the pos member is initialized and this method
+//                     must take care of the rest
+// RETURNS    : TRUE if step is allowed and the node should be added to
+//              the open list, FALSE if step is not allowed
 ////////////////////////////////////////////////////////////////////////
 
-PathNode *Path::GetCost( const Unit *u, const PathNode *pn_from, const Point &to ) const {
-  unsigned short sofar = pn_from->cost, state;
-  short cost = map->MoveCost( u, pn_from->pos, to, state );
+bool Path::AddNode( const Unit *u, const PathNode &from, PathNode &to ) const {
+  unsigned short obst;
+  short cost = map->MoveCost( u, from.pos, to.pos, obst );
+  bool rc = false;
 
-  if ( state != 0 ) {
-    if ( !map->GetMapObject( to ) || (to == end) )
-      cost = MAX( u->Moves() - sofar, cost );
+  if ( obst != 0 ) {
+    if ( !map->GetMapObject( to.pos ) || (to.pos == end) )
+      cost = MAX( u->Moves() - from.cost, cost );
     else cost = -1;
   }
 
   if ( cost >= 0 ) {
-    PathNode *ret = new PathNode;
-    if ( ret ) {
-      ret->pos = to;
-      ret->cost = sofar + cost;
-      return ret;
-    }
+    to.cost = from.cost + cost;
+    to.eta = ETA( to.pos );
+    rc = true;
   }
-  return NULL;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : Path::MarkBuffer
-// DESCRIPTION: When a hex has been checked and was found ok to move
-//              onto, this hex must be properly marked in the path
-//              buffer and possibly in the list of nodes, too.
-// PARAMETERS : next - node which is to be added to the open list
-//              dir  - direction of the step to reach next node
-//              open - list of open nodes
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void Path::MarkBuffer( PathNode *next, Direction dir, List &open ) const {
-  unsigned short index = map->Hex2Index( next->pos );
-  PathNode *p;
-
-  // if the new hex is not marked in path yet, mark it and put it in openlist
-  if ( path[index] == -1 ) {
-    path[index] = ReverseDir( dir );
-    open.AddTail( next );
-  } else {
-    // try to find the node in openlist and replace it if new cost is smaller
-    p = static_cast<PathNode *>( open.Head() );
-    while ( p ) {
-      if ( p->pos == next->pos ) {   // found the node. shall we replace it?
-        if ( p->cost > next->cost ) {
-          p->Remove();
-          delete p;
-          path[index] = ReverseDir( dir );
-          open.AddTail( next );
-          next = NULL;
-        }
-        break;
-      } else p = static_cast<PathNode *>( p->Next() );
-    }
-    delete next;
-  }
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : Path::RecordPath
+// NAME       : Path::FinalizePath
 // DESCRIPTION: Check whether a path has been found, prepare the buffer
 //              and set the return code of the pathfinder.
 // PARAMETERS : u    - unit searching the path
 //              last - final path node
-//              off  - maximum allowed distance to destination
 // RETURNS    : cost of the path in turns for the unit (approximately),
 //              or -1 if no valid path was found
 ////////////////////////////////////////////////////////////////////////
 
-short Path::RecordPath( const Unit *u, const PathNode *last,
-                        unsigned char off ) const {
+short Path::FinalizePath( const Unit *u, const PathNode &last ) const {
   short turns = -1;
-  Point p = last->pos;
-  if ( Distance( p, end ) <= off ) {
+  Point p = last.pos;
+  if ( Distance( p, end ) <= deviation ) {
     // correct the path
     // move back to beginning
     short index = map->Hex2Index( p );
@@ -326,131 +314,78 @@ short Path::RecordPath( const Unit *u, const PathNode *last,
     }
 
     if ( u->Type()->Speed() == 0 ) turns = 100;
-    else turns = (last->cost + u->Type()->Speed() - 1) / u->Type()->Speed();
+    else turns = (last.cost + u->Type()->Speed() - 1) / u->Type()->Speed();
   }
   return turns;
 }
 
 
-
 // MOVE SHADER - this is used to create the buffer for shading of
 //               illegal moves when selecting a unit
 //
 //
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MoveShader::GetBestNode
-// DESCRIPTION: For the shader the order of nodes doesn't matter. Nodes
-//              with a too high cost are not added to the list, and all
-//              others must be processed anyway. Therefore we always
-//              use the list head.
-// PARAMETERS : open - list of open nodes
-// RETURNS    : best node
+// NAME       : MoveShader::ETA
+// DESCRIPTION: Estimate the cost to the destination hex. This is used
+//              as an heuristic for the path finding algorithm.
+// PARAMETERS : current - current location
+// RETURNS    : estimated cost to destination
 ////////////////////////////////////////////////////////////////////////
 
-PathNode *MoveShader::GetBestNode( const List &open ) const {
-  PathNode *p1, *p2;
-
-  p1 = static_cast<PathNode *>( open.Head() );
-  p2 = static_cast<PathNode *>( p1->Next() );
-  while ( p2 ) {
-    if ( p2->cost < p1->cost ) p1 = p2;
-    p2 = static_cast<PathNode *>( p2->Next() );
-  }
-  return p1;
+inline unsigned short MoveShader::ETA( const Point &p ) const {
+  return 1;
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MoveShader::StopSearch
-// DESCRIPTION: The shader never stops the serach when there are nodes
+// DESCRIPTION: The shader never stops the search while there are nodes
 //              left in the open list.
 // PARAMETERS : next - path node to be checked next if search is not
 //                     cancelled
-//              off  - minimum distance to end to abort
 // RETURNS    : TRUE if search aborted, FALSE otherwise
 ////////////////////////////////////////////////////////////////////////
 
-bool MoveShader::StopSearch( const PathNode *next, unsigned char off ) const {
+inline bool MoveShader::StopSearch( const PathNode &next ) const {
   return false;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MoveShader::GetCost
+// NAME       : MoveShader::AddNode
 // DESCRIPTION: Calculate the cost for the unit to move from one hex to
 //              another.
-// PARAMETERS : u       - unit
-//              pn_from - path node for source hex
-//              to      - destination hex (adjacent to from)
-// RETURNS    : next PathNode or NULL on illegal path
+// PARAMETERS : u    - unit
+//              from - path node for source hex
+//              to   - destination path node
+// RETURNS    : TRUE if step is allowed and the node should be added to
+//              the open list, FALSE if step is not allowed
 ////////////////////////////////////////////////////////////////////////
 
-PathNode *MoveShader::GetCost( const Unit *u, const PathNode *pn_from, const Point &to ) const {
-  unsigned short sofar = pn_from->cost;
-  unsigned short state;
-  short cost = map->MoveCost( u, pn_from->pos, to, state );
+bool MoveShader::AddNode( const Unit *u, const PathNode &from, PathNode &to ) const {
+  unsigned short obst;
+  short cost = map->MoveCost( u, from.pos, to.pos, obst );
+  bool rc = false;
 
-  if ( cost > u->Moves() - sofar ) cost = -1;
-  else if ( state != 0 ) cost = u->Moves() - sofar;
+  if ( cost > u->Moves() - from.cost ) cost = -1;
+  else if ( obst != 0 ) cost = u->Moves() - from.cost;
 
   if ( cost >= 0 ) {
-    PathNode *ret = new PathNode;
-    if ( ret ) {
-      ret->pos = to;
-      ret->cost = pn_from->cost + cost;
-      return ret;
-    }
+    to.cost = from.cost + cost;
+    to.eta = ETA( to.pos );
+    rc = true;
   }
-  return NULL;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MoveShader::MarkBuffer
-// DESCRIPTION: When a hex has been checked and was found ok to move
-//              onto, this hex must be properly marked in the path
-//              buffer and possibly in the list of nodes, too.
-// PARAMETERS : next - node which is to be added to the open list
-//              dir  - direction of the step to reach next node
-//              open - list of open nodes
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void MoveShader::MarkBuffer( PathNode *next, Direction dir, List &open ) const {
-  unsigned short index = map->Hex2Index( next->pos );
-  PathNode *p;
-
-  // if the new hex is not marked in path yet, mark it and put it in openlist
-  if ( path[index] == -1 ) {
-    path[index] = 1;
-    open.AddTail( next );
-  } else {
-    // try to find the node in openlist and replace it if new cost is smaller
-    p = static_cast<PathNode *>( open.Head() );
-    while ( p ) {
-      if ( p->pos == next->pos ) {   // found the node. shall we replace it?
-        if ( p->cost > next->cost ) {
-          p->Remove();
-          delete p;
-          open.AddTail( next );
-          next = NULL;
-        }
-        break;
-      } else p = static_cast<PathNode *>( p->Next() );
-    }
-    delete next;
-  }
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : MoveShader::RecordPath
+// NAME       : MoveShader::FinalizePath
 // DESCRIPTION: The shader doesn't have a path to prepare, but we must
 //              check for valid targets which must not be shaded.
 // PARAMETERS : u    - unit searching the path
 //              last - final path node
-//              off  - maximum allowed distance to destination
 // RETURNS    : always 1
 ////////////////////////////////////////////////////////////////////////
 
-short MoveShader::RecordPath( const Unit *u, const PathNode *last,
-                              unsigned char off ) const {
+short MoveShader::FinalizePath( const Unit *u, const PathNode &last ) const {
   for ( Unit *target = static_cast<Unit *>(units.Head());
         target; target = static_cast<Unit *>(target->Next()) ) {
     if ( u->CanHit( target ) )
@@ -463,27 +398,32 @@ short MoveShader::RecordPath( const Unit *u, const PathNode *last,
 
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MinesweeperShader::GetCost
+// NAME       : MinesweeperShader::AddNode
 // DESCRIPTION: See whether there's a mine to be swept on a hex next to
 //              the mine-sweeper unit. Enemy mines can only be cleared
 //              if there is no other enemy unit (excluding mines) on a
 //              field adjacent to the mine.
-// PARAMETERS : u       - mine-sweeper unit
-//              pn_from - path node for source hex
-//              to      - destination hex (adjacent to from)
-// RETURNS    : next PathNode or NULL on illegal path (no mine)
+// PARAMETERS : u    - mine-sweeper unit
+//              from - path node for source hex
+//              to   - destination path node (adjacent to from)
+// RETURNS    : TRUE if step is allowed and the node should be added to
+//              the open list, FALSE if step is not allowed
 ////////////////////////////////////////////////////////////////////////
 
-PathNode *MinesweeperShader::GetCost( const Unit *u, const PathNode *pn_from, const Point &to ) const {
-  if ( NextTo( u->Position(), to ) && (u->Terrain() & map->TerrainTypes(to)) ) {
-    Unit *m = map->GetUnit( to );
+bool MinesweeperShader::AddNode( const Unit *u, const PathNode &from,
+                                 PathNode &to ) const {
+  bool rc = false;
+
+  if ( NextTo( u->Position(), to.pos ) &&
+     (u->Terrain() & map->TerrainTypes(to.pos)) ) {
+    Unit *m = map->GetUnit( to.pos );
 
     if ( m && m->IsMine() ) {
       bool enemy = false;
 
       if ( m->Owner() != u->Owner() ) {
         Point adj[6];
-        map->GetNeighbors( to, adj );
+        map->GetNeighbors( to.pos, adj );
         for ( int i = NORTH; (i <= NORTHWEST) && !enemy; ++i ) {
           if ( adj[i].x != -1 ) {
             Unit *e = map->GetUnit( adj[i] );
@@ -493,50 +433,24 @@ PathNode *MinesweeperShader::GetCost( const Unit *u, const PathNode *pn_from, co
       }
 
       if ( !enemy ) {
-        PathNode *ret = new PathNode;
-        if ( ret ) {
-          ret->pos = to;
-          ret->cost = 0;
-          return ret;
-        }
+        to.cost = 0;
+        to.eta = ETA( to.pos );
+        rc = true;
       }
     }
   }
-  return NULL;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : MinesweeperShader::MarkBuffer
-// DESCRIPTION: When a hex has been checked and was found ok to move
-//              onto, this hex must be properly marked in the path
-//              buffer.
-// PARAMETERS : next - node which is to be added to the open list
-//              dir  - direction of the step to reach next node
-//              open - list of open nodes
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void MinesweeperShader::MarkBuffer( PathNode *next, Direction dir, List &open ) const {
-  unsigned short index = map->Hex2Index( next->pos );
-
-  // if the new hex is not marked in path yet, mark it and put it in openlist
-  if ( path[index] == -1 ) {
-    path[index] = 1;
-    open.AddTail( next );
-  } else delete next;
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : MinesweeperShader::RecordPath
+// NAME       : MinesweeperShader::FinalizePath
 // DESCRIPTION: The shader doesn't have a path to prepare.
 // PARAMETERS : u    - unit searching the path
 //              last - final path node
-//              off  - maximum allowed distance to destination
 // RETURNS    : always 1
 ////////////////////////////////////////////////////////////////////////
 
-short MinesweeperShader::RecordPath( const Unit *u, const PathNode *last,
-                              unsigned char off ) const {
+short MinesweeperShader::FinalizePath( const Unit *u, const PathNode &last ) const {
   path[map->Hex2Index( u->Position() )] = 1;
   return 1;
 }
@@ -568,19 +482,19 @@ TransPath::TransPath( Map *map, const Transport *trans, signed char *buffer ) :
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : TransPath::GetCost
+// NAME       : TransPath::AddNode
 // DESCRIPTION: Calculate the cost for the unit to move from one hex to
 //              another.
-// PARAMETERS : u       - unit
-//              pn_from - path node for source hex
-//              to      - destination hex (adjacent to from)
-// RETURNS    : next PathNode or NULL on illegal move
+// PARAMETERS : u    - unit
+//              from - path node for source hex
+//              to   - destination path node (adjacent to from)
+// RETURNS    : TRUE if step is allowed and the node should be added to
+//              the open list, FALSE if step is not allowed
 ////////////////////////////////////////////////////////////////////////
 
-PathNode *TransPath::GetCost( const Unit *u, const PathNode *pn_from, const Point &to ) const {
-  const Point &from = pn_from->pos;
-  unsigned short sofar = pn_from->cost;
-  const Unit *cur = (pn_from->switched ? u : t);
+bool TransPath::AddNode( const Unit *u, const PathNode &from, PathNode &to ) const {
+  const Unit *cur = (from.switched ? u : t);
+  bool rc = false;
 
   // important for all checks: on a TransPath the path is checked in the
   // direction Transport->Destination where destination may be the unit
@@ -588,40 +502,39 @@ PathNode *TransPath::GetCost( const Unit *u, const PathNode *pn_from, const Poin
   // least one step before reaching the destination (switched == true)
   // for any path to be legal.
   short cost = -1;
-  const TerrainType *type = map->HexType( to );
+  const TerrainType *type = map->HexType( to.pos );
   bool hexok = (type->tt_type & cur->Terrain()) != 0;
 
-  Unit *block = map->GetUnit( to );
-  if ( (to != end) || pn_from->switched ) {
+  Unit *block = map->GetUnit( to.pos );
+  if ( (to.pos != end) || from.switched ) {
     if ( block ) {
-      if ( block->IsTransport() && (to == end) && pn_from->switched &&
+      if ( block->IsTransport() && (to.pos == end) && from.switched &&
            !cur->IsSheltered() && static_cast<Transport *>(block)->Allow(cur) )
-        cost = MAX( cur->Moves() - sofar, MCOST_MIN );
+        cost = MAX( cur->Moves() - from.cost, MCOST_MIN );
       else if ( hexok ) cost = MCOST_UNIT;
     } else {
       unsigned short state;
-      cost = map->MoveCost( cur, from, to, state );
+      cost = map->MoveCost( cur, from.pos, to.pos, state );
       if ( state != 0 ) {
-        if ( sofar >= cur->Moves() ) cost = cur->Moves();
-        else cost = MAX( cur->Moves() - sofar, cost );
+        if ( from.cost >= cur->Moves() ) cost = cur->Moves();
+        else cost = MAX( cur->Moves() - from.cost, cost );
       }
     }
   }
 
-  PathNode *pn_to = NULL;
   if ( cost != -1 ) {
-    pn_to = new PathNode();
-    pn_to->pos = to;
-    pn_to->cost = sofar + cost;
-    pn_to->switched = pn_from->switched;
-  } else if ( !pn_from->switched ) {
+    to.cost = from.cost + cost;
+    to.eta = ETA( to.pos );
+    to.switched = from.switched;
+    rc = true;
+  } else if ( !from.switched ) {
     // let's see if we can continue here when switching to the other unit
-    PathNode pn_tmp( *pn_from );
-    pn_tmp.switched = true;
-    pn_to = GetCost( u, &pn_tmp, to );
+    PathNode from2 = from;
+    from2.switched = to.switched = true;
+    rc = AddNode( u, from2, to );
   }
 
-  return pn_to;
+  return rc;
 }
 
 ////////////////////////////////////////////////////////////////////////
