@@ -270,15 +270,14 @@ bool Unit::CouldHit( const Unit *enemy ) const {
 // NAME       : Unit::Repair
 // DESCRIPTION: Repair the unit (completely). This will get the
 //              group size of the unit back to the maximum, but will
-//              also lower its experience by one point per two
-//              "rookies".
+//              also lower its experience by one point per "rookie".
 // PARAMETERS : -
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 void Unit::Repair( void ) {
   if ( u_group < MAX_GROUP_SIZE ) {
-    u_xp = MAX( 0, u_xp - (MAX_GROUP_SIZE - u_group)/2 );
+    u_xp = MAX( 0, u_xp - (MAX_GROUP_SIZE - u_group) );
     u_group = MAX_GROUP_SIZE;
     SetFlags( U_DONE );	// can't move this turn
   }
