@@ -37,14 +37,9 @@
 #define ICON_MINI_BUILD1_Y	46
 #define ICON_MINI_BUILD2_X	108
 #define ICON_MINI_BUILD2_Y	46
-#define ICON_MINI_TRANSFER1_X	123
-#define ICON_MINI_TRANSFER1_Y	46
-#define ICON_MINI_TRANSFER2_X	123
-#define ICON_MINI_TRANSFER2_Y	46
 
 #define CW_MODE_NORMAL		0
 #define CW_MODE_PRODUCTION	1
-#define CW_MODE_TRANSFER	2
 
 // node class used by the UnitListWidget
 class ULWNode : public Node {
@@ -78,11 +73,9 @@ private:
 // button IDs for the container hook
 #define CH_BUTTON_REPAIR	0
 #define CH_BUTTON_BUILD		1
-#define CH_BUTTON_TRANSFER	2
 
-#define CH_BUTTON_REPAIR_CONFIRM 3
-#define CH_LIST_UNITS		4
-#define CH_SLIDER_CRYSTALS	5
+#define CH_BUTTON_REPAIR_CONFIRM 2
+#define CH_LIST_UNITS		3
 
 // widget IDs for the UnitLoadWindow
 #define ULW_WIDGET_LIST		1
@@ -277,16 +270,6 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
         buttons[CH_BUTTON_BUILD] = btn;
       }
     }
-
-    btn = new ButtonWidget( CH_BUTTON_TRANSFER, xoff, yoff,
-                  ICON_MINI_WIDTH + 4, ICON_MINI_HEIGHT + 4,
-                  WIDGET_STYLE_GFX, NULL, this );
-    btn->SetImage( icons, Rect( ICON_MINI_TRANSFER1_X, ICON_MINI_TRANSFER1_Y,
-                                ICON_MINI_WIDTH, ICON_MINI_HEIGHT ),
-                          Rect( ICON_MINI_TRANSFER2_X, ICON_MINI_TRANSFER2_Y,
-                                ICON_MINI_WIDTH, ICON_MINI_HEIGHT ) );
-    btn->SetHook( this );
-    buttons[CH_BUTTON_TRANSFER] = btn;
   }
 
   height = h - lfont->Height() - ICON_MINI_HEIGHT - 55;
@@ -415,9 +398,8 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
         }
       }
 
-      // disable transfer and repair buttons
+      // disable repair button
       if ( buttons[CH_BUTTON_REPAIR] ) buttons[CH_BUTTON_REPAIR]->Disable();
-      buttons[CH_BUTTON_TRANSFER]->Disable();
 
       // display new list in list widget
       DrawBack( unitinfo );
@@ -427,86 +409,18 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
           &m->GetUnitSet(), &m->GetTerrainSet(), this, unitinfo );
       Show();
     }
-  } else if ( newmode == CW_MODE_TRANSFER ) {       // must be in CW_MODE_NORMAL
-    // disable repair and build buttons and list widget
-    if ( buttons[CH_BUTTON_REPAIR] ) buttons[CH_BUTTON_REPAIR]->Disable();
-    if ( buttons[CH_BUTTON_BUILD] ) buttons[CH_BUTTON_BUILD]->Disable();
-    listwidget->Disable();
+  } else {       // switch back to CW_MODE_NORMAL from CW_MODE_PRODUCTION
+    // destroy production list
+    while ( !build.IsEmpty() ) delete build.RemHead();
 
-    // clear unit info area
+    // re-enable repair and button
+    if ( buttons[CH_BUTTON_REPAIR] ) buttons[CH_BUTTON_REPAIR]->Enable();
+
+    // display normal list in widget
     DrawBack( unitinfo );
+    listwidget->SwitchList( &normal, -1 );
     UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
-
-    // create new widgets
-    Transport *t = static_cast<Transport *>( static_cast<ULWNode *>(listwidget->Selected())->unit );
-    char buf[8];
-    unsigned short min, max;
-    min = MAX( 0, c->Crystals() - (t->MaxCrystals() - t->Crystals()) );
-    max = MIN( c->MaxCrystals(), c->Crystals() + t->Crystals() );
-
-    slider = new SliderWidget( CH_SLIDER_CRYSTALS, unitinfo.x + 10,
-                       unitinfo.y + (unitinfo.h - DEFAULT_SLIDER_SIZE) / 2,
-                       unitinfo.w - 20, DEFAULT_SLIDER_SIZE,
-                       min, max, c->Crystals(), max - min + 1,
-                       WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
-    slider->SetHook( this );
-    stringwidgets[0] = new StringWidget( 0, slider->LeftEdge(),
-                       slider->TopEdge() + slider->Height() + 5,
-                       sfont->Width() * 4 + 8, sfont->Height() + 8,
-                       itoa( t->Crystals(), buf ), 4,
-                       WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
-    stringwidgets[1] = new StringWidget( 0,
-                       slider->LeftEdge() + slider->Width() - stringwidgets[0]->Width(),
-                       stringwidgets[0]->TopEdge(), stringwidgets[0]->Width(),
-                       stringwidgets[0]->Height(), itoa( c->Crystals(), buf ), 4,
-                       WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
-
-    slider->Draw();
-    stringwidgets[0]->Draw();
-    stringwidgets[1]->Draw();
-
-    // draw images
-    m->GetUnitSet().DrawTile( t->BaseImage(), this, slider->LeftEdge(),
-                   slider->TopEdge() - ICON_HEIGHT - 3, unitinfo );
-    crystals_icon.Draw( this, slider->LeftEdge() + slider->Width() - ICON_WIDTH,
-                   slider->TopEdge() - ICON_HEIGHT - 3 );
     Show();
-  } else {                                          // switch back to CW_MODE_NORMAL...
-    if ( mode == CW_MODE_PRODUCTION ) {             // ...from CW_MODE_PRODUCTION
-      // destroy production list
-      while ( !build.IsEmpty() ) delete build.RemHead();
-
-      // re-enable repair and transfer buttons
-      if ( buttons[CH_BUTTON_REPAIR] ) buttons[CH_BUTTON_REPAIR]->Enable();
-      buttons[CH_BUTTON_TRANSFER]->Enable();
-
-      // display normal list in widget
-      DrawBack( unitinfo );
-      listwidget->SwitchList( &normal, -1 );
-      UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
-      Show();
-    } else if ( mode == CW_MODE_TRANSFER ) {          // ...from CW_MODE_TRANSFER
-      // delete transfer stuff
-      RemoveWidget( slider );
-      RemoveWidget( stringwidgets[0] );
-      RemoveWidget( stringwidgets[1] );
-      delete slider;
-      delete stringwidgets[0];
-      delete stringwidgets[1];
-
-      // redraw unit info
-      DrawBack( unitinfo );
-      UnitInfoWindow::DrawUnitInfo(
-          static_cast<ULWNode *>(listwidget->Selected())->type->ID(),
-          &m->GetUnitSet(), &m->GetTerrainSet(), this, unitinfo );
-      DrawCrystals();
-
-      // re-enable repair and build buttons and list widget
-      if ( buttons[CH_BUTTON_REPAIR] ) buttons[CH_BUTTON_REPAIR]->Enable();
-      if ( buttons[CH_BUTTON_BUILD] ) buttons[CH_BUTTON_BUILD]->Enable();
-      listwidget->Enable();
-      Show();
-    }
   }
 
   mode = newmode;
@@ -592,16 +506,6 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
   case CH_BUTTON_BUILD:
     SwitchMode( (Mode() == CW_MODE_NORMAL) ? CW_MODE_PRODUCTION : CW_MODE_NORMAL );
     break;
-  case CH_BUTTON_TRANSFER: {
-    ULWNode *n = static_cast<ULWNode *>(listwidget->Selected());
-    if ( n ) {
-      Unit *u = n->unit;
-      if ( u->IsTransport() ) {
-        SwitchMode( (Mode() == CW_MODE_NORMAL) ? CW_MODE_TRANSFER : CW_MODE_NORMAL );
-      } else new NoteWindow( MSG(MSG_TRANSFER), MSG(MSG_ERR_NO_TRANSPORTER),
-                             WIN_CLOSE_ESC, view );
-    }
-    break; }
 
   case CH_BUTTON_REPAIR_CONFIRM: {
     Audio::PlaySfx( Audio::SND_GAM_REPAIR, 0 );
@@ -632,19 +536,6 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
       last_selected = node;
     } else if ( node ) rc = SelectNode( node );
     break; }
-  case CH_SLIDER_CRYSTALS: {
-    // current slider level is amount of crystals for the first exchange partner
-    Transport *t = static_cast<Transport *>( static_cast<ULWNode *>(listwidget->Selected())->unit );
-    short current = slider->Level();
-    short current2 = t->Crystals() + c->Crystals() - current;
-
-    char buf[8];
-    stringwidgets[1]->SetString( itoa(current, buf), true );
-    stringwidgets[0]->SetString( itoa(current2, buf), true );
-
-    c->SetCrystals( current );
-    t->SetCrystals( current2 );
-    break; }
   }
 
   return rc;
@@ -689,7 +580,7 @@ UnitLoadWindow::UnitLoadWindow( Transport &t, UnitContainer &c,
     }
   }
 
-  if ( !have_units ) return;
+  if ( !Opened() ) return;
 
   // create list widget
   wd_list = new UnitListWidget(
@@ -731,8 +622,7 @@ UnitLoadWindow::UnitLoadWindow( Transport &t, UnitContainer &c,
   wd_slider = new SliderWidget( ULW_WIDGET_CRYSTALS, unitinfo.x + ICON_WIDTH,
            wd_crystals->TopEdge() + (wd_crystals->Height() - DEFAULT_SLIDER_SIZE) / 2,
            wd_crystals->LeftEdge() - unitinfo.x - ICON_WIDTH - 5, DEFAULT_SLIDER_SIZE,
-           0, max, crystals, unitinfo.w / (max+1),
-           WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
+           0, max, crystals, max/5, WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
   wd_slider->SetHook( this );
 
   Draw();
@@ -813,7 +703,7 @@ GUI_Status UnitLoadWindow::WidgetActivated( Widget *widget, Window *win ) {
         unsigned short max =
           MIN( t.MaxCrystals() - ((fullslots - (crystals+9)/10) * 10),
                c.Crystals() + t.Crystals() );
-        wd_slider->Adjust( 0, max, unitinfo.w / (max+1) );
+        wd_slider->Adjust( 0, max, max/5 );
         wd_slider->Draw();
         wd_slider->Show();
       } else Audio::PlaySfx( Audio::SND_GUI_ERROR, 0 );
@@ -840,7 +730,7 @@ GUI_Status UnitLoadWindow::WidgetActivated( Widget *widget, Window *win ) {
           node = static_cast<ULWNode *>(node->Next()) ) {
       if ( node->selected ) {
         c.RemoveUnit( node->unit );
-        node->unit->SetFlags( U_MOVED );
+        node->unit->SetFlags( U_MOVED|U_DONE );
         t.InsertUnit( node->unit );
       }
     }
