@@ -23,6 +23,7 @@
 #include "edwindow.h"
 #include "extwindow.h"
 #include "extwindow2.h"
+#include "gamewindow.h"
 #include "filewindow.h"
 #include "gfxwidget.h"
 #include "fileio.h"
@@ -66,8 +67,8 @@ const short EdWindow::B_ID_QUIT       = B_ID_MENU_BASE + 16;
 
 EdWindow::EdWindow( const string &mapdir, View *view ) :
           Window( view->x, view->y, view->w, view->h, 0, view ),
-          panel( view->w - GFX_WIDTH * 4 - DEFAULT_SLIDER_SIZE - 20,
-                 view->y, GFX_WIDTH * 4 + DEFAULT_SLIDER_SIZE + 20, view->h ),
+          panel( view->w - DEFAULT_TILE_WIDTH * 4 - DEFAULT_SLIDER_SIZE - 20,
+                 view->y, DEFAULT_TILE_WIDTH * 4 + DEFAULT_SLIDER_SIZE + 20, view->h ),
           mv( this,
               Rect( DEFAULT_SLIDER_SIZE, 0,
               w - panel.w - DEFAULT_SLIDER_SIZE, h - DEFAULT_SLIDER_SIZE),
@@ -76,17 +77,16 @@ EdWindow::EdWindow( const string &mapdir, View *view ) :
   mission = NULL;
   selected = -1;
 
-  unsigned short height = MIN( GFX_HEIGHT * 6 + 4, h / 3 - 10 );
+  unsigned short height = MIN( DEFAULT_TILE_HEIGHT * 6 + 4, h / 3 - 10 );
   unit_wd = new UnitWidget( L_ID_UNITS, panel.x + 5,
                             panel.y + panel.h - 6 - height,
-                            panel.w - 10, height, 0, NULL,
-                            this, mv, NULL, GFX_WIDTH, GFX_HEIGHT );
+                            panel.w - 10, height, 0, NULL, this, NULL );
   unit_wd->SetHook( this );
 
   tile_wd = new TileWidget( L_ID_TILES, unit_wd->x,
                             panel.y + sfont->Height() + 15,
                             unit_wd->w, panel.h - height - sfont->Height() - 30,
-                            0, NULL, this, mv, NULL, GFX_WIDTH, GFX_HEIGHT );
+                            0, NULL, this, NULL );
   tile_wd->SetHook( this );
 
   ud_wd = new SliderWidget( S_ID_VERT, 0, 0, DEFAULT_SLIDER_SIZE, mv.Height(),
@@ -305,9 +305,9 @@ void EdWindow::LeftMouseButton( const Point &hex ) {
     if ( map->GetMapObject(hex) ) new NoteWindow( "Error", "Hex occupied.", 0, view );
     else {
       UnitSet *us = map->GetUnitSet();
-      const UnitType *ut = us->GetUnitInfo( selected < us->NumUT() ?
-                                            selected : selected - us->NumUT() );
-      mission->CreateUnit( ut, selected < us->NumUT() ? PLAYER_ONE : PLAYER_TWO, hex );
+      const UnitType *ut = us->GetUnitInfo( selected < us->NumTiles() ?
+                                            selected : selected - us->NumTiles() );
+      mission->CreateUnit( ut, selected < us->NumTiles() ? PLAYER_ONE : PLAYER_TWO, hex );
     }
   }
 
@@ -550,6 +550,7 @@ GUI_Status EdWindow::WidgetActivated( Widget *widget, Window *win ) {
       break;
 
     case B_ID_UNIT_INFO:
+      new UnitInfoWindow( map->GetUnit( selected_hex )->Type()->ID(), *map, view );
       break;
     case B_ID_UNIT_EDIT: {
       EdUnitWindow *euw = new EdUnitWindow( *(map->GetUnit( selected_hex )), *mission, view );
@@ -633,7 +634,7 @@ void EdWindow::ShowContextMenu( const Point &clicked ) {
 
       if ( u ) {  // unit sub-menu
         menu->AddMenu( 0, 0, "_Unit" );
-        menu->AddItem( 1, B_ID_UNIT_INFO, WIDGET_DISABLED, "_Info" );  // FIXME
+        menu->AddItem( 1, B_ID_UNIT_INFO, 0, "_Info" );
         menu->AddItem( 1, B_ID_UNIT_EDIT, 0, "_Edit..." );
         menu->AddItem( 1, B_ID_UNIT_DEL, 0, "_Delete" );
       }
@@ -730,14 +731,16 @@ void EdWindow::SetNewMission( Mission *ms ) {
   mv.SetMap( &map );
   mv.SetCursor( Point(0,0) );
 
-  tile_wd->SetTerrainSet( map.GetTerrainSet(), GFX_WIDTH, GFX_HEIGHT );
-  unit_wd->SetUnitSet( map.GetUnitSet(), GFX_WIDTH, GFX_HEIGHT );
+  tile_wd->SetTerrainSet( map.GetTerrainSet() );
+  unit_wd->SetUnitSet( map.GetUnitSet() );
 
-  short max = MAX( map.Height() * GFX_HEIGHT + GFX_OVERLAP_Y - mv.Height(), 0 );
+  short max = MAX( map.Height() * mv.TileHeight() +
+                   mv.TileShiftY() - mv.Height(), 0 );
   ud_wd->Adjust( 0, max, mv.Height() );
   ud_wd->ScrollTo( 0 );
 
-  max = MAX( map.Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - mv.Width(), 0 );
+  max = MAX( map.Width() * (mv.TileWidth() - mv.TileShiftX()) +
+             mv.TileShiftX() - mv.Width(), 0 );
   lr_wd->Adjust( 0, max, mv.Width() );
   lr_wd->ScrollTo( 0 );
 
