@@ -90,6 +90,13 @@ const short EdEventSetTimerWindow::B_ID_EVENT_OK = 2;
 const short EdEventSetTimerWindow::B_ID_OFFSET = 3;
 const short EdEventSetTimerWindow::N_ID_TIME = 4;
 
+const short EdTrigHaveCrystalsWindow::B_ID_SHOP = 1;
+const short EdTrigHaveCrystalsWindow::B_ID_SHOP_OK = 2;
+const short EdTrigHaveCrystalsWindow::B_ID_PLAYER = 3;
+const short EdTrigHaveCrystalsWindow::N_ID_CRYSTALS = 4;
+const short EdTrigHaveCrystalsWindow::B_ID_MOREORLESS = 5;
+const short EdTrigHaveCrystalsWindow::B_ID_TRANSPORTS = 6;
+
 const short EdTrigHaveShopWindow::B_ID_SHOP = 1;
 const short EdTrigHaveShopWindow::B_ID_SHOP_OK = 2;
 const short EdTrigHaveShopWindow::B_ID_PLAYER = 3;
@@ -311,6 +318,8 @@ GUI_Status EdEventGenericWindow::WidgetActivated( Widget *widget, Window *win )
     switch ( event.Trigger() ) {
     case ETRIGGER_HAVE_BUILDING:
       new EdTrigHaveShopWindow( event, mission, view ); break;
+    case ETRIGGER_HAVE_CRYSTALS:
+      new EdTrigHaveCrystalsWindow( event, mission, view ); break;
     case ETRIGGER_HAVE_UNIT:
       new EdTrigHaveUnitWindow( event, mission, view ); break;
     case ETRIGGER_TIMER:
@@ -1235,6 +1244,115 @@ GUI_Status EdEventSetTimerWindow::WidgetActivated( Widget *widget, Window *win )
 }
 
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdTrigHaveCrystalsWindow::EdTrigHaveCrystalsWindow
+// DESCRIPTION: This window allows configuration of a HAVE_CRYSTALS
+//              trigger.
+// PARAMETERS : event   - event to edit
+//              mission - current mission
+//              view    - pointer to the window's view
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+EdTrigHaveCrystalsWindow::EdTrigHaveCrystalsWindow( Event &event,
+                      Mission &mission, View *view ) :
+      Window(WIN_CENTER, view), event(event), mission(mission) { 
+  SetSize( MIN(sfont->Width() * 30 + 15, view->Width()),
+           sfont->Height() * 4 + 50 );
+
+  Widget *wd;
+  unsigned short wdx, wdh = sfont->Height() + 8;
+  static const char *moreless_labels[] = { ">=", "<", 0 };
+
+  wd = new ButtonWidget( B_ID_SHOP, 5, 5,
+       sfont->Width() * 4 + 10, wdh, 0, "_Shop", this );
+  wd->SetHook( this );
+
+  Building *b = mission.GetBuildingByID( event.GetTData(2) );
+  wdx = wd->LeftEdge() + wd->Width() + 5;
+  t_shop = new StringWidget( 0, wdx, 5, w - wdx - 5, wdh,
+           b ? b->Name() : "All", 30, WIDGET_STR_CONST,
+           NULL, this );
+
+  wdx = sfont->TextWidth("Player") + 10;
+  wd = new CycleWidget( B_ID_PLAYER, wdx, wd->Height() + 10,
+       sfont->Width() * 3 + 20, wdh, 0, "_Player",
+       event.GetTData(1), player2_labels, this );
+  wd->SetHook( this );
+
+  wdx = w - 5 - sfont->Width() * 4 - 20;
+  wd = new CycleWidget( B_ID_MOREORLESS, wdx, wd->TopEdge(),
+       sfont->Width() * 4 + 20, wdh, 0, NULL,
+       event.GetTData(0) < 0 ? 1 : 0, moreless_labels, this );
+  wd->SetHook( this );
+
+  wdx = wd->LeftEdge() - 5 - sfont->Width() * 6;
+  wd = new NumberWidget( N_ID_CRYSTALS, wdx, wd->TopEdge(),
+       sfont->Width() * 6, wdh, ABS(event.GetTData(0)),
+       1, 5000, WIDGET_ALIGN_LEFT, "_Crystals", this );
+  wd->SetHook( this );
+
+  t_transports = new CheckboxWidget( B_ID_TRANSPORTS,
+       5, wd->TopEdge() + wd->Height() + 5,
+       DEFAULT_CBW_SIZE, DEFAULT_CBW_SIZE,
+       event.GetTData(2) == -2,
+       WIDGET_STYLE_GFX|WIDGET_STYLE_NOBORDER|WIDGET_ALIGN_RIGHT|
+       (event.GetTData(2) >= 0 ? WIDGET_DISABLED : 0),
+       "_Include Transports", this );
+  t_transports->SetHook( this );
+
+  new ButtonWidget( GUI_CLOSE, 1, h - wdh - 1, w - 2, wdh,
+                    WIDGET_DEFAULT, "_OK", this );
+
+  Draw();
+  Show();
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : EdTrigHaveCrystalsWindow::WidgetActivated
+// DESCRIPTION: React to user actions.
+// PARAMETERS : widget - activated widget
+//              win    - window containing the widget
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status EdTrigHaveCrystalsWindow::WidgetActivated( Widget *widget, Window *win ) {
+
+  switch( widget->ID() ) {
+  case B_ID_SHOP:
+    TLWListBuilder::BuildShopList( mission.GetBuildings(), tlist );
+    new ListSelectWindow( "Select shop", tlist,
+        event.GetTData(2), true, this, B_ID_SHOP_OK, view );
+    break;
+  case B_ID_SHOP_OK: {
+    TLWNode *n = static_cast<ListSelectWindow *>(win)->Selected();
+    event.SetTData( 2, n ? n->ID() : -1 );
+    t_shop->SetString( n ? n->Name() : "All" );
+    tlist.Clear();
+
+    if ( !n ) {
+      t_transports->Enable();
+      if ( t_transports->Clicked() ) event.SetTData( 2, -2 );
+    } else t_transports->Disable();
+    t_transports->Draw();
+    Show();
+    break; }
+  case B_ID_PLAYER:
+    event.SetTData( 1, static_cast<CycleWidget *>(widget)->GetValue() );
+    break;
+  case N_ID_CRYSTALS:
+    event.SetTData( 0, SIGN(event.GetTData(0)) * static_cast<NumberWidget *>(widget)->Number() );
+    break;
+  case B_ID_MOREORLESS:
+    event.SetTData( 0, event.GetTData( 0 ) * -1 );
+    break;
+  case B_ID_TRANSPORTS:
+    event.SetTData( 2, widget->Clicked() ? -2 : -1 );
+  }
+
+  return GUI_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : EdTrigHaveShopWindow::EdTrigHaveShopWindow
 // DESCRIPTION: This window allows configuration of a HAVE_BUILDING
