@@ -237,6 +237,28 @@ private:
   Mission &mission;
 };
 
+class EdTrigHaveCrystalsWindow : public Window, public WidgetHook {
+public:
+  EdTrigHaveCrystalsWindow( Event &event, Mission &mission, View *view );
+
+private:
+  GUI_Status WidgetActivated( Widget *button, Window *win );
+
+  static const short B_ID_SHOP;
+  static const short B_ID_SHOP_OK;
+  static const short B_ID_PLAYER;
+  static const short N_ID_CRYSTALS;
+  static const short B_ID_MOREORLESS;
+  static const short B_ID_TRANSPORTS;
+
+  StringWidget *t_shop;
+  CheckboxWidget *t_transports;
+  TLWList tlist;
+
+  Event &event;
+  Mission &mission;
+};
+
 class EdTrigHaveShopWindow : public Window, public WidgetHook {
 public:
   EdTrigHaveShopWindow( Event &event, Mission &mission, View *view );
