@@ -960,7 +960,7 @@ EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
   wd->SetHook( this );
 
   yoff += 5 + wdh;
-  if ( u.IsTransport() ) {
+  if ( u.IsTransport() && !u.IsSheltered() ) {
     xoff = 10 + sfont->TextWidth("Crystals");
     // maximum number of crystals should be
     // (mission.StorageLeft(u) + (u.Crystals() + 9)/10) * 10,
@@ -972,11 +972,9 @@ EdUnitWindow::EdUnitWindow( Unit &unit, Mission &mission, View *view ) :
                            WIDGET_ALIGN_LEFT, "_Crystals", this );
     wd->SetHook( this );
 
-    if ( !u.IsSheltered() ) {
-      wd = new ButtonWidget( B_ID_UNITS, w/2 + 5, yoff, (w - 20)/2, wdh,
-                             0, "_Units...", this ),
-      wd->SetHook( this );
-    }
+    wd = new ButtonWidget( B_ID_UNITS, w/2 + 5, yoff, (w - 20)/2, wdh,
+                           0, "_Units...", this ),
+    wd->SetHook( this );
   }
 
   wd = new ButtonWidget( B_ID_OK, 1, h - 1 - wdh, w - 2, wdh,
