@@ -257,34 +257,27 @@ GUI_Status GfxWidget::KeyDown( const SDL_keysym &key ) {
 //              flags  - widget flags (see widget.h for details)
 //              title  - widget title (currently unused)
 //              window - widget parent window
-//              mv     - map view
 //              ts     - terrain set used (may be NULL)
-//              gfxw   - tile width
-//              gfxh   - tile height
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 TileWidget::TileWidget( short id, short x, short y, unsigned short w,
     unsigned short h, unsigned short flags, const char *title,
-    Window *window, MapView &mv, TerrainSet *ts, unsigned short gfxw,
-    unsigned short gfxh ) :
-    GfxWidget( id, x, y, w, h, flags, title, window ), mv(mv) {
-  SetTerrainSet( ts, gfxw, gfxh );
+    Window *window, TerrainSet *ts ) :
+    GfxWidget( id, x, y, w, h, flags, title, window ) {
+  SetTerrainSet( ts );
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : TileWidget::SetTerrainSet
 // DESCRIPTION: Attach a different terrain set to this widget.
-// PARAMETERS : ts   - terrain set to be used
-//              gfxw - tile width
-//              gfxh - tile height
+// PARAMETERS : ts - terrain set to be used
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void TileWidget::SetTerrainSet( TerrainSet *ts, unsigned short gfxw,
-                                                unsigned short gfxh ) {
+void TileWidget::SetTerrainSet( TerrainSet *ts ) {
   tset = ts;
-  if ( ts ) Init( gfxw, gfxh, ts->NumTT() );
+  if ( ts ) Init( ts->TileWidth(), ts->TileHeight(), ts->NumTiles() );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -314,34 +307,27 @@ short TileWidget::GetImage( unsigned short type ) const {
 //              flags  - widget flags (see widget.h for details)
 //              title  - widget title (currently unused)
 //              window - widget parent window
-//              mv     - map view
 //              us     - unit set used (may be NULL)
-//              gfxw   - tile width
-//              gfxh   - tile height
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 UnitWidget::UnitWidget( short id, short x, short y, unsigned short w,
     unsigned short h, unsigned short flags, const char *title,
-    Window *window, MapView &mv, UnitSet *us, unsigned short gfxw,
-    unsigned short gfxh ) :
-    GfxWidget( id, x, y, w, h, flags, title, window ), mv(mv) {
-  SetUnitSet( us, gfxw, gfxh );
+    Window *window, UnitSet *us ) :
+    GfxWidget( id, x, y, w, h, flags, title, window ) {
+  SetUnitSet( us );
 }
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitWidget::SetUnitSet
 // DESCRIPTION: Attach a different unit set to this widget.
-// PARAMETERS : us   - unit set to be used
-//              gfxw - tile width
-//              gfxh - tile height
+// PARAMETERS : us - unit set to be used
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void UnitWidget::SetUnitSet( UnitSet *us, unsigned short gfxw,
-                                          unsigned short gfxh ) {
+void UnitWidget::SetUnitSet( UnitSet *us ) {
   uset = us;
-  if ( us ) Init( gfxw, gfxh, us->NumUT() * 2 );
+  if ( us ) Init( us->TileWidth(), us->TileHeight(), us->NumTiles() * 2 );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -354,9 +340,9 @@ void UnitWidget::SetUnitSet( UnitSet *us, unsigned short gfxw,
 short UnitWidget::GetImage( unsigned short type ) const {
   short rc = -1;
   if ( uset ) {
-    const UnitType *ut = uset->GetUnitInfo(type % uset->NumUT());
+    const UnitType *ut = uset->GetUnitInfo(type % uset->NumTiles());
     if ( ut ) {
-      if ( type < uset->NumUT() ) rc = ut->Image();
+      if ( type < uset->NumTiles() ) rc = ut->Image();
       else rc = ut->Image() + 9;      /* player 2, facing southwards */
     }
   }
