@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,6 +25,10 @@
 #include <string.h>
 #include "SDL.h"
 
+#ifndef _WIN32_WCE
+# include <time.h>
+#endif
+
 #include "edwindow.h"
 #include "main.h"
 #include "mission.h"
@@ -45,6 +49,8 @@ static void save_settings( View *display );
 int main( int argc, char **argv ) {
   struct EdOptions options;
 
+  srand( time(0) ); // init random number generator
+
   parse_options( argc, argv, options );
 
   // create options, save games, and maps directories if possible
