@@ -32,7 +32,7 @@ const char *event_labels[] = {
       "Manipulate Event", "Research", "Set Hex", "Set Timer", 0 };
 const char *etrigger_labels[] = {
       "Timer", "Unit Destruction", "Shop Owner", "Unit Owner",
-      "Unit Position", "Handicap", 0 };
+      "Unit Position", "Handicap", "Have Crystals", 0 };
 
 #define P1_COLOR_LIGHT	0x00FEA604		// yellow
 #define P1_COLOR_SHADOW	0x0056360C		// brown
@@ -857,6 +857,14 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     }
     break;
 
+  case ETRIGGER_HAVE_CRYSTALS:
+    if ( (e.GetTData(2) >= 0) && !GetBuildingByID(e.GetTData(2)) ) {
+      ++errs;
+      errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
+             << ") event targets non-existing shop with ID " << e.GetTData(2) << ".\n\n";
+    }
+    break;
+
   case ETRIGGER_HAVE_UNIT:
     if ( (e.GetTData(0) < 0) || !GetUnitByID(e.GetTData(0)) ) {
       ++errs;
@@ -1166,6 +1174,11 @@ Event::Event( unsigned char id, unsigned char type, unsigned char trigger ) {
     e_tdata[1] = PLAYER_ONE;
     e_tdata[2] = 0;
     break;
+  case ETRIGGER_HAVE_CRYSTALS:
+    e_tdata[0] = 100;         // >= 100 crystals
+    e_tdata[1] = PLAYER_ONE;  // owned by first player
+    e_tdata[2] = -1;          // check all shops
+    break;
   case ETRIGGER_HAVE_BUILDING:
   case ETRIGGER_HAVE_UNIT:
     e_tdata[0] = -1;          // check shop/unit -1
@@ -1277,6 +1290,12 @@ int Event::Export( ofstream &file, const Map &map ) const {
     file << "towner = " << e_tdata[1] + 1 << '\n';
     file << "ttime = " << e_tdata[2] << '\n';
     break;
+  case ETRIGGER_HAVE_CRYSTALS:
+    file << "havecrystals\n";
+    file << "tcrystals = " << e_tdata[0] << '\n';
+    file << "towner = " << e_tdata[1] + 1 << '\n';
+    file << "tbuilding = " << e_tdata[2] << '\n';
+    break;
   case ETRIGGER_HAVE_UNIT:
     file << "haveunit\n";
     file << "tunit = " << e_tdata[0] << '\n';
