@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -226,12 +226,13 @@ void MessageWindow::Draw( void ) {
 //              msg   - single-line message to be printed (may be NULL)
 //              pass  - password to check the user input against; if
 //                      this is NULL any input will be accepted
+//              abort - whether dialog can be cancelled
 //              view  - pointer to the window's view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 PasswordWindow::PasswordWindow( const char *title, const char *msg,
-                        const char *pass, View *view ) :
+                        const char *pass, bool abort, View *view ) :
             Window( WIN_CENTER, view ) {
   this->title = title;
   password = NULL;
@@ -252,12 +253,17 @@ PasswordWindow::PasswordWindow( const char *title, const char *msg,
                              NULL, 7, WIDGET_ALIGN_TOP|WIDGET_STR_PASSWORD,
                              msg, this );
 
+  if ( abort ) width = w/2 - 2;
+  else width = w - 2;
+
   Widget *wd = new ButtonWidget( GUI_CLOSE, 1, h - sfont->Height() - 9,
-                                 w/2 - 2, sfont->Height() + 8, WIDGET_DEFAULT,
+                                 width, sfont->Height() + 8, WIDGET_DEFAULT,
                                  MSG(MSG_B_OK), this );
-  wd = new ButtonWidget( GUI_RESTART,
-                         wd->LeftEdge() + wd->Width() + 1, wd->TopEdge(),
-                         wd->Width(), wd->Height(), 0, MSG(MSG_B_CANCEL), this );
+  if ( abort ) {
+    new ButtonWidget( GUI_RESTART,
+                      wd->LeftEdge() + wd->Width() + 1, wd->TopEdge(),
+                      wd->Width(), wd->Height(), 0, MSG(MSG_B_CANCEL), this );
+  }
 
   Draw();
   Show();
