@@ -396,6 +396,8 @@ string get_home_dir( void ) {
   if ( length == 0 ) return "";
   else return dir;
 # endif
+#elif defined __BEOS__
+  return "./";
 #else
   return getenv( "HOME" );
 #endif
