@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -90,12 +90,13 @@ public:
   File( const string &path ) : fh(0), name(path) {}
   ~File( void ) { if (fh) Close(); }
 
-  bool Open( const char *opts );
+  bool Open( const char *mode, bool compressed = true );
+  bool OpenData( const char *mode, const string &subdir = "", bool compressed = true );
   void Close( void );
   static bool Exists( const string &name );
 
   int Read( void *buffer, int size ) const
-      { return SDL_RWread(fh, buffer, size, 1); }
+      { return SDL_RWread(fh, buffer, 1, size); }
   unsigned char Read8( void ) const;
   unsigned short Read16( void ) const { return SDL_ReadLE16(fh); }
   unsigned long Read32( void ) const { return SDL_ReadLE32(fh); }
@@ -132,6 +133,7 @@ string get_home_levels_dir( void );
 void create_config_dir( void );
 
 string file_part( const string &path );
+void append_path( string &path, const string &sub );
 void append_path_delim( string &path );
 void make_dir( const char *dir );
 
