@@ -156,7 +156,7 @@ unsigned short Font::FitText( const char *str, unsigned short width,
         else {
           // add single characters until we hit the wall
           do {
-            line.push_back( str[pos++] );
+            line += str[pos++];
           } while ( TextWidth( line.c_str() ) <= width );
           return pos - 1;
         }
@@ -164,10 +164,10 @@ unsigned short Font::FitText( const char *str, unsigned short width,
 
       if ( (str[pos] == '\n') || (str[pos] == '\0') ) return pos + 1;
       lastspace = pos;
-      line.push_back( str[pos] );
+      line += str[pos];
     }
 
-    else line.push_back( str[pos] );
+    else line += str[pos];
 
   } while ( str[pos++] != '\0' );
 
