@@ -24,6 +24,65 @@
 #include "fileio.h"
 #include "misc.h"
 
+#define FOG_ALPHA 128
+#define IMG_FOG   1
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : TileSet::Load
+// DESCRIPTION: Load a tile set from a file. This only reads the image
+//              surface(s) and associated data.
+// PARAMETERS : file    - file descriptor
+//              setname - name of the requested tile set. This is
+//                        actually only used when saving a mission to
+//                        a file
+// RETURNS    : 0 on success, non-zero on error
+////////////////////////////////////////////////////////////////////////
+
+int TileSet::Load( File &file, const char *setname ) {
+
+  int rc = tiles.LoadImageData( file, true );
+  if ( !rc ) {
+    tiles.DisplayFormat();
+    name.assign( setname );
+  }
+
+  return rc;
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : TileSet::DrawTile
+// DESCRIPTION: Draw a tile image to a surface.
+// PARAMETERS : n    - image number
+//              dest - destination surface
+//              px   - horizontal offset on surface
+//              py   - vertical offset on surface
+//              clip - clipping rectangle
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void TileSet::DrawTile( unsigned short n, Surface *dest,
+                        short px, short py, const Rect &clip ) const {
+  // set up destination
+  Rect dstrect( px, py, TileWidth(), TileHeight() );
+  if ( dstrect.x + dstrect.w < clip.x ) return;
+  if ( dstrect.y + dstrect.h < clip.y ) return;
+  if ( dstrect.x >= clip.x + clip.w ) return;
+  if ( dstrect.y >= clip.y + clip.h ) return;
+
+  // set up source
+  Rect srcrect;
+  unsigned short gfx_per_line = tiles.Width() / TileWidth();
+  srcrect.x = (n % gfx_per_line) * TileWidth();
+  srcrect.y = (n / gfx_per_line) * TileHeight();
+  srcrect.w = dstrect.w;
+  srcrect.h = dstrect.h;
+
+  // clip and blit to surface
+  dstrect.ClipBlit( srcrect, clip );
+  tiles.LowerBlit( dest, srcrect, dstrect.x, dstrect.y );
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitSet::UnitSet
 // DESCRIPTION: Initialize a new unit set.
@@ -32,7 +91,7 @@
 ////////////////////////////////////////////////////////////////////////
 
 UnitSet::UnitSet( void ) {
-  num_ut = num_sfx = 0;
+  num_sfx = 0;
   ut = NULL;
   sfx = NULL;
   portraits = NULL;
@@ -62,7 +121,7 @@ UnitSet::~UnitSet( void ) {
 ////////////////////////////////////////////////////////////////////////
 
 const UnitType *UnitSet::GetUnitInfo( unsigned short utid ) const {
-  if ( utid < num_ut ) return &ut[utid];
+  if ( utid < num_tiles ) return &ut[utid];
   return NULL;
 }
 
@@ -81,10 +140,10 @@ SoundEffect *UnitSet::GetSound( unsigned short sfxid ) const {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitSet::Load
 // DESCRIPTION: Load a unit set from a file.
-// PARAMETERS : file     - file descriptor
-//              setname  - name of the requested unit set. This is
-//                         actually only used when saving a mission to
-//                         a file
+// PARAMETERS : file    - file descriptor
+//              setname - name of the requested unit set. This is
+//                        actually only used when saving a mission to
+//                        a file
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
@@ -92,11 +151,11 @@ int UnitSet::Load( File &file, const char *setname ) {
   int rc = -1;
 
   if ( file.Read32() == FID_UNITSET ) {
-    num_ut = file.Read16();
+    num_tiles = file.Read16();
     num_sfx = file.Read8();
 
     if ( !LoadSfx( file ) && !LoadUnitTypes( file ) &&
-         !tiles.LoadImageData( file, true ) ) {
+         !TileSet::Load( file, setname ) ) {
       unsigned char num_portraits = file.Read8();
       if ( num_portraits ) {
         portraits = new Surface [num_portraits];
@@ -107,8 +166,6 @@ int UnitSet::Load( File &file, const char *setname ) {
         }
       }
 
-      tiles.DisplayFormat();
-      name.assign( setname );
       rc = 0;
     }
   }
@@ -124,12 +181,12 @@ int UnitSet::Load( File &file, const char *setname ) {
 ////////////////////////////////////////////////////////////////////////
 
 int UnitSet::LoadUnitTypes( File &file ) {
-  ut = new UnitType [num_ut];
+  ut = new UnitType [num_tiles];
   if ( !ut ) return -1;
 
   int rc = 0;
 
-  for ( int i = 0; i < num_ut; ++i ) {
+  for ( int i = 0; i < num_tiles; ++i ) {
     if ( ut[i].Load( file, this ) ) {
       rc = -1;
       break;
@@ -143,7 +200,7 @@ int UnitSet::LoadUnitTypes( File &file ) {
   if ( rc == -1 ) {
       delete [] ut;
       ut = NULL;
-      num_ut = 0;
+      num_tiles = 0;
       return -1;
   }
 
@@ -186,7 +243,7 @@ int UnitSet::LoadSfx( File &file ) {
 void UnitSet::SetLocale( const string &lang ) {
   unit_names.SetDefaultLanguage( lang );
 
-  for ( int i = 0; i < num_ut; ++i )
+  for ( int i = 0; i < num_tiles; ++i )
     ut[i].SetName( unit_names.GetMsg( i ) );
 }
 
@@ -223,10 +280,10 @@ TerrainSet::~TerrainSet( void ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : TerrainSet::Load
 // DESCRIPTION: Load a terrain set from a file.
-// PARAMETERS : file     - file descriptor
-//              setname  - name of the requested terrain set. This is
-//                         actually only used when saving a mission to
-//                         a file
+// PARAMETERS : file    - file descriptor
+//              setname - name of the requested terrain set. This is
+//                        actually only used when saving a mission to
+//                        a file
 // RETURNS    : 0 on success, non-zero on error
 ////////////////////////////////////////////////////////////////////////
 
@@ -234,16 +291,23 @@ int TerrainSet::Load( File &file, const char *setname ) {
   int rc = -1;
 
   if ( file.Read32() == FID_TERRAINSET ) {
-    num_tt = file.Read16();
+    num_tiles = file.Read16();
 
     for ( int i = 0; i < 10; ++i )
       classid[i] = file.Read16();
 
-    if ( !LoadTerrainTypes( file ) && !tiles.LoadImageData( file, true ) ) {
-      tiles.DisplayFormat();
-      name.assign( setname );
+    if ( !LoadTerrainTypes( file ) && !TileSet::Load( file, setname ) )
+      // create the fog surface. This is kept separate from the tiles
+      // surface so that we don't need to modify the alpha value each
+      // time we want fog
+      fog.Create( TileWidth(), TileHeight(), 16, SDL_SWSURFACE );
+      fog.SetAlpha( FOG_ALPHA, SDL_SRCALPHA );
+      fog.SetColorKey( Color(CF_COLOR_WHITE) );
+      fog.DisplayFormat();
+      fog.Flood( Color(CF_COLOR_WHITE) );
+      DrawTile( IMG_FOG, &fog, 0, 0, fog );
+
       rc = 0;
-    }
   }
 
   return rc;
@@ -257,7 +321,7 @@ int TerrainSet::Load( File &file, const char *setname ) {
 ////////////////////////////////////////////////////////////////////////
 
 const TerrainType *TerrainSet::GetTerrainInfo( unsigned short ttid ) const {
-  if ( ttid < num_tt ) return &tt[ttid];
+  if ( ttid < num_tiles ) return &tt[ttid];
   return NULL;
 }
 
@@ -269,20 +333,34 @@ const TerrainType *TerrainSet::GetTerrainInfo( unsigned short ttid ) const {
 ////////////////////////////////////////////////////////////////////////
 
 int TerrainSet::LoadTerrainTypes( File &file ) {
-  tt = new TerrainType [num_tt];
+  tt = new TerrainType [num_tiles];
   if ( !tt ) return -1;
 
-  for ( int i = 0; i < num_tt; ++i ) {
+  for ( int i = 0; i < num_tiles; ++i ) {
     if ( tt[i].Load( file ) ) {
       delete [] tt;
       tt = NULL;
-      num_tt = 0;
+      num_tiles = 0;
       return -1;
     }
   }
   return 0;
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : TerrainSet::DrawFog
+// DESCRIPTION: Draw the fog hex to a surface.
+// PARAMETERS : dest - destination surface
+//              px   - horizontal offset on surface
+//              py   - vertical offset on surface
+//              clip - clipping rectangle
+// RETURNS    : -
+////////////////////////////////////////////////////////////////////////
+
+void TerrainSet::DrawFog( Surface *dest, short px, short py, const Rect &clip ) const {
+  fog.Blit( dest, fog, px, py );
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitType::Load
 // DESCRIPTION: Load a unit type definition from a file.
