@@ -37,7 +37,6 @@ MapView::MapView( Surface *display, const Rect &bounds, unsigned short flags ) :
   surface = display;
 
   map = NULL;
-  utiles = ttiles = NULL;
   shader_map = NULL;
 
   SetFlags( flags );
@@ -52,11 +51,10 @@ MapView::MapView( Surface *display, const Rect &bounds, unsigned short flags ) :
 ////////////////////////////////////////////////////////////////////////
 
 void MapView::InitOffsets( void ) {
-  maxx = MAX( 0, map->Width() * (GFX_WIDTH - GFX_OVERLAP_X) + GFX_OVERLAP_X - w );
-  maxy = MAX( 0, map->Height() * GFX_HEIGHT + GFX_OVERLAP_Y - h );
+  maxx = MAX( 0, map->Width() * (TileWidth() - TileShiftX()) + TileShiftX() - w );
+  maxy = MAX( 0, map->Height() * TileHeight() + TileShiftY() - h );
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::Resize
 // DESCRIPTION: Set a new viewport size and position.
@@ -78,12 +76,11 @@ void MapView::Resize( const Rect &bounds ) {
   else CenterOnHex( cursor );
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::SetMap
 // DESCRIPTION: Assign another map to this viewport.
 // PARAMETERS : map - map to display; the map must have already had
-//                    a level set assigned.
+//                    a level set assigned
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
@@ -92,31 +89,12 @@ void MapView::SetMap( Map *map ) {
 
   cursor = Point( -1, -1 );
   cursor_image = IMG_CURSOR_IDLE;
-
   this->map = map;
-  UnitSet *uset = map->GetUnitSet();
-  if ( uset ) utiles = uset->GetTiles();
-  TerrainSet *tset = map->GetTerrainSet();
-  if ( tset ) {
-    ttiles = tset->GetTiles();
-
-    // create the fog surface. This is kept separate from the tiles
-    // surface so that we don't need to modify the alpha value each
-    // time we want fog. That way we can also use SDL_RLEACCEL on
-    // both surfaces which would otherwise slow us down cosiderably
-    fog.Create( GFX_WIDTH, GFX_HEIGHT, 16, SDL_HWSURFACE );
-    fog.SetAlpha( FOG_ALPHA, SDL_SRCALPHA );
-    fog.SetColorKey( Color(CF_COLOR_WHITE) );
-    fog.DisplayFormat();
-    fog.Flood( Color(CF_COLOR_WHITE) );
-    DrawTerrain( IMG_SHADOW, &fog, 0, 0, fog );
-  }
 
   InitOffsets();
   shader_map = new signed char [map->Width() * map->Height()];
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::Enable
 // DESCRIPTION: Set map offsets back to 0 and enable map display.
@@ -140,7 +118,6 @@ void MapView::Disable( void ) {
   SetFlags( MV_DISABLE );
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::Draw
 // DESCRIPTION: Draw a part of the map onto the window surface.
@@ -183,14 +160,14 @@ void MapView::DrawMap( short x, short y, unsigned short w,
   Point hex;
 
   for ( tx = hx1; tx <= hx2; ++tx ) {
-    sx = tx * (GFX_WIDTH - GFX_OVERLAP_X) - x + this->x;
+    sx = tx * (TileWidth() - TileShiftX()) - x + this->x;
 
-    if ( tx & 1 ) yoff = GFX_OVERLAP_Y - y;
+    if ( tx & 1 ) yoff = TileShiftY() - y;
     else yoff = -y;
     yoff += this->y;
 
     for ( ty = hy1; ty <= hy2; ++ty ) {
-      sy = ty * GFX_HEIGHT + yoff;
+      sy = ty * TileHeight() + yoff;
       hex = Point( tx, ty );
 
       // draw the terrain image
@@ -217,7 +194,6 @@ void MapView::DrawMap( short x, short y, unsigned short w,
   }
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::Pixel2Hex
 // DESCRIPTION: Convert viewport coordinates to hex coordinates.
@@ -233,8 +209,24 @@ int MapView::Pixel2Hex( short px, short py, Point &hex ) const {
   if ( Contains( px, py ) ) {
     short hx = px + curx - x, hy = py + cury - y;
 
-    hex.x = (hx - GFX_OVERLAP_X/2) / (GFX_WIDTH - GFX_OVERLAP_X);
-    hex.y = (hy - ((hex.x & 1) ? GFX_OVERLAP_Y : 0)) / GFX_HEIGHT;
+    hex.x = hx / (TileWidth() - TileShiftX());
+    hex.y = (hy + (hex.x & 1) * TileShiftY()) / TileHeight() - (hex.x & 1);
+
+    // calculate pixel position relative to selected hex
+    hx %= TileWidth() - TileShiftX();
+    hy -= hex.y * TileHeight() + (hex.x & 1) * TileShiftY();
+
+    const Surface &mask = map->GetTerrainSet()->HexMask();
+    if ( mask.GetPixel( hx, hy ) == mask.GetColorKey() ) {
+      if ( hx < (TileWidth() / 2) ) --hex.x;
+      else ++hex.x;
+
+      if ( hy < (TileHeight() / 2) ) {
+        if ( hex.x & 1 ) --hex.y; // odd columns
+      } else {
+        if ( !(hex.x & 1) ) ++hex.y; // even columns
+      }
+    }
 
     if ( map->Contains( hex ) ) return 0;
   }
@@ -252,8 +244,8 @@ int MapView::Pixel2Hex( short px, short py, Point &hex ) const {
 ////////////////////////////////////////////////////////////////////////
 
 Point MapView::Hex2Pixel( const Point &hex ) const {
-  return Point( hex.x * (GFX_WIDTH - GFX_OVERLAP_X) - curx + x,
-                hex.y * GFX_HEIGHT + ((hex.x & 1) ? GFX_OVERLAP_Y : 0) - cury + y );
+  return Point( hex.x * (TileWidth() - TileShiftX()) - curx + x,
+                hex.y * TileHeight() + (hex.x & 1) * TileShiftY() - cury + y );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -267,8 +259,8 @@ Point MapView::Hex2Pixel( const Point &hex ) const {
 bool MapView::HexVisible( const Point &hex ) const {
   Point p = Hex2Pixel( hex );  // get absolute pixel values
 
-  return( (p.x >= x) && (p.x <= x + w - GFX_WIDTH) &&
-          (p.y >= y) && (p.y <= y + h - GFX_HEIGHT) );
+  return( (p.x >= x) && (p.x <= x + w - TileWidth()) &&
+          (p.y >= y) && (p.y <= y + h - TileHeight()) );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -284,7 +276,7 @@ Rect MapView::UpdateHex( const Point &hex ) {
 
   Point p = Hex2Pixel( hex );  // get absolute position
 
-  Rect clip( p.x, p.y, GFX_WIDTH, GFX_HEIGHT );   // create clip rect
+  Rect clip( p.x, p.y, TileWidth(), TileHeight() );   // create clip rect
   clip.Clip( *this );
 
   // draw the terrain image
@@ -311,55 +303,6 @@ Rect MapView::UpdateHex( const Point &hex ) {
   return clip;
 }
 
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : MapView::DrawHex
-// DESCRIPTION: Draw a hex image to a surface.
-// PARAMETERS : n    - image number
-//              src  - source surface
-//              dest - destination surface
-//              px   - horizontal offset on surface
-//              py   - vertical offset on surface
-//              clip - clipping rectangle
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void MapView::DrawHex( unsigned short n, Surface *src, Surface *dest,
-                       short px, short py, const Rect &clip ) {
-  // set up destination
-  Rect dstrect( px, py, GFX_WIDTH, GFX_HEIGHT );
-  if ( dstrect.x + dstrect.w < clip.x ) return;
-  if ( dstrect.y + dstrect.h < clip.y ) return;
-  if ( dstrect.x >= clip.x + clip.w ) return;
-  if ( dstrect.y >= clip.y + clip.h ) return;
-
-  // set up source
-  Rect srcrect;
-  unsigned short gfx_per_line = src->Width() / GFX_WIDTH;
-  srcrect.x = (n % gfx_per_line) * GFX_WIDTH;
-  srcrect.y = (n / gfx_per_line) * GFX_HEIGHT;
-  srcrect.w = GFX_WIDTH;
-  srcrect.h = GFX_HEIGHT;
-
-  // clip and blit to surface
-  dstrect.ClipBlit( srcrect, clip );
-  src->LowerBlit( dest, srcrect, dstrect.x, dstrect.y );
-}
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : MapView::DrawFog
-// DESCRIPTION: Draw the fog hex to a surface.
-// PARAMETERS : dest - destination surface
-//              px   - horizontal offset on surface
-//              py   - vertical offset on surface
-//              clip - clipping rectangle
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-void MapView::DrawFog( Surface *dest, short px, short py, const Rect &clip ) const {
-  fog.Blit( dest, fog, px, py );
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::DrawUnitHealth
 // DESCRIPTION: Draw a unit health bar.
@@ -378,9 +321,13 @@ void MapView::DrawUnitHealth( unsigned char health, Surface *dest,
   else if ( health >= 3 ) hcol = Color(0xFE,0xAA,0x04); // yellow
   else hcol = Color(0xFE,0x26,0x26);                    // red
 
-  Rect outer( px + GFX_OVERLAP_X + (GFX_WIDTH - 2 * GFX_OVERLAP_X - 8)/2,
-              py + GFX_HEIGHT - 5, 8, 4 );
-  Rect inner( outer.x + 1, outer.y + 1, health, 2 );
+  unsigned short hp = (TileWidth() - 2 * TileShiftX() - 4) /
+                      (MAX_GROUP_SIZE + 2);
+  unsigned short barw = hp * MAX_GROUP_SIZE + 2;
+
+  Rect outer( px + (TileWidth() - barw) / 2,
+              py + TileHeight() - hp - 4, barw, hp + 3 );
+  Rect inner( outer.x + 1, outer.y + 1, health * hp, hp + 1 );
 
   outer.Clip( clip );
   inner.Clip( clip );
@@ -399,19 +346,18 @@ void MapView::DrawUnitHealth( unsigned char health, Surface *dest,
 void MapView::CenterOnHex( const Point &hex ) {
   Point p = Hex2Pixel( hex );
 
-  short off = p.x + (GFX_WIDTH - w) / 2 + curx;
+  short off = p.x + (TileWidth() - w) / 2 + curx;
   if ( off > maxx ) curx = maxx;
   else if ( off < 0 ) curx = 0;
   else curx = off;
 
-  off = p.y + (GFX_HEIGHT - h) / 2 + cury;
+  off = p.y + (TileHeight() - h) / 2 + cury;
   if ( off > maxy ) cury = maxy;
   else if ( off < 0 ) cury = 0;
   else cury = off;
   Draw();
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::MinXHex
 // DESCRIPTION: Get the leftmost visible hex number.
@@ -423,7 +369,7 @@ void MapView::CenterOnHex( const Point &hex ) {
 
 unsigned short MapView::MinXHex( short x ) const {
   if ( x == -1 ) x = curx;
-  return( MAX( (x - GFX_OVERLAP_X) / (GFX_WIDTH - GFX_OVERLAP_X), 0 ) );
+  return MAX( (x - TileShiftX()) / (TileWidth() - TileShiftX()), 0 );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -437,7 +383,7 @@ unsigned short MapView::MinXHex( short x ) const {
 
 unsigned short MapView::MinYHex( short y ) const {
   if ( y == -1 ) y = cury;
-  return( MAX( (y - GFX_OVERLAP_Y) / GFX_HEIGHT, 0 ) );
+  return MAX( (y - TileShiftY()) / TileHeight(), 0 );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -455,7 +401,7 @@ unsigned short MapView::MaxXHex( short x, unsigned short w ) const {
     x = curx;
     w = Width();
   }
-  return( MIN( (x + w) / (GFX_WIDTH - GFX_OVERLAP_X), map->Width() - 1 ) );
+  return MIN( (x + w) / (TileWidth() - TileShiftX()), map->Width() - 1 );
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -473,10 +419,9 @@ unsigned short MapView::MaxYHex( short y, unsigned short h ) const {
     y = cury;
     h = Height();
   }
-  return( MIN( (y + h) / GFX_HEIGHT, map->Height() - 1 ) );
+  return MIN( (y + h) / TileHeight(), map->Height() - 1 );
 }
 
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : MapView::CheckScroll
 // DESCRIPTION: Look at the current cursor position and map offsets and
@@ -491,12 +436,14 @@ bool MapView::CheckScroll( void ) {
 
   Point p = Hex2Pixel( cursor );
 
-  if ( (curx > 0) && (p.x <= x + GFX_WIDTH) ) p.x = -w / 2;
-  else if ( (curx < maxx) && (w + x - p.x - GFX_WIDTH <= GFX_WIDTH) ) p.x = w / 2;
+  if ( (curx > 0) && (p.x <= x + TileWidth()) ) p.x = -w / 2;
+  else if ( (curx < maxx) && (w + x - p.x - TileWidth() <= TileWidth()) )
+    p.x = w / 2;
   else p.x = 0;
 
-  if ( (cury > 0) && (p.y <= y + GFX_HEIGHT) ) p.y = -h / 2;
-  else if ( (cury < maxy) && (h + y - p.y - GFX_HEIGHT <= GFX_HEIGHT) ) p.y = h / 2;
+  if ( (cury > 0) && (p.y <= y + TileHeight()) ) p.y = -h / 2;
+  else if ( (cury < maxy) && (h + y - p.y - TileHeight() <= TileHeight()) )
+    p.y = h / 2;
   else p.y = 0;
 
   if ( p.x || p.y ) {
