@@ -80,6 +80,45 @@ Color Surface::GetColorKey( void ) const {
   return Color( r, g, b );
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Surface::GetPixel
+// DESCRIPTION: Read the pixel color at a specified position.
+// PARAMETERS : x - horiontal pixel position
+//              y - vertical pixel position
+// RETURNS    : pixel color
+////////////////////////////////////////////////////////////////////////
+
+Color Surface::GetPixel( unsigned short x, unsigned short y ) const {
+  Uint8 r, g, b, bpp = s_surface->format->BytesPerPixel;
+  Uint32 col = 0;
+
+  {
+    SurfaceLock lck( this );
+    Uint8 *pixel = ((Uint8 *)s_surface->pixels) + y * s_surface->pitch + x * bpp;
+
+    switch ( bpp ) {
+    case 1: 
+      col = *pixel;
+      break;
+    case 2:
+      col = *((Uint16 *)pixel);
+      break;
+    case 3:
+      if ( SDL_BYTEORDER == SDL_LIL_ENDIAN )
+        col = pixel[0] | (pixel[1] << 8) | (pixel[2] << 16);
+      else
+        col = (pixel[0] << 16) | (pixel[1] << 8) | pixel[2];
+      break;
+    case 4:
+      col = *((Uint32 *)pixel);
+      break;
+    }
+  }
+
+  SDL_GetRGB( col, s_surface->format, &r, &g, &b );
+  return Color( r, g, b );
+}
+
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Surface::DisplayFormat
 // DESCRIPTION: Convert surface format to display format.
@@ -230,7 +269,8 @@ int Surface::FillRectAlpha( short x, short y, unsigned short w, unsigned short h
   // get offset into surface buffer
   bpp = fmt->BytesPerPixel;
   if ( bpp < 2 ) return FillRect( x, y, w, h, col );   // not supported
-  if ( SDL_MUSTLOCK( s_surface ) && (SDL_LockSurface(s_surface) < 0) ) return -1;
+
+  SurfaceLock lck( this );
 
   for ( int py = y; py < y + h; ++py ) {
     bits = ((Uint8 *)s_surface->pixels) + py * s_surface->pitch + x * bpp;
@@ -245,9 +285,10 @@ int Surface::FillRectAlpha( short x, short y, unsigned short w, unsigned short h
           pixel = *((Uint16 *)(bits));
           break;
         case 3:
-          if ( SDL_BYTEORDER == SDL_LIL_ENDIAN ) {
+          if ( SDL_BYTEORDER == SDL_LIL_ENDIAN )
             pixel = bits[0] + (bits[1] << 8) + (bits[2] << 16);
-          } else pixel = (bits[0] << 16) + (bits[1] << 8) + bits[2];
+          else
+            pixel = (bits[0] << 16) + (bits[1] << 8) + bits[2];
           break;
         case 4:
           pixel = *((Uint32 *)(bits));
@@ -268,7 +309,6 @@ int Surface::FillRectAlpha( short x, short y, unsigned short w, unsigned short h
       bits += bpp;
     }
   }
-  if ( SDL_MUSTLOCK( s_surface ) ) SDL_UnlockSurface( s_surface );
 
   return 0;
 }
