@@ -381,7 +381,8 @@ const char *StringWidget::String( void ) const {
 void StringWidget::Draw( void ) {
   Rect clip;
   surface->DrawBack( *this );
-  surface->DrawBox( *this, BOX_RECESSED );
+  if ( !(flags & WIDGET_STYLE_NOBORDER ) )
+    surface->DrawBox( *this, BOX_RECESSED );
   surface->GetClipRect( clip );
   surface->SetClipRect( strbox );
 
