@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -283,6 +283,24 @@ GUI_Status View::FetchEvent( SDL_Event &event ) {
   do {
     if ( SDL_WaitEvent( &event ) ) {
 
+      // try to aggregate several mouse motion events to prevent getting flooded
+      if ( event.type == SDL_MOUSEMOTION ) {
+        SDL_Event next;
+        Uint32 start = SDL_GetTicks();
+
+        do {
+          int got = SDL_PeepEvents( &next, 1, SDL_GETEVENT, SDL_MOUSEMOTIONMASK );
+
+          if ( got > 0 ) {
+            event.motion.x = next.motion.x;
+            event.motion.y = next.motion.y;
+            event.motion.xrel += next.motion.xrel;
+            event.motion.yrel += next.motion.yrel;
+          } else break;
+
+        } while ( SDL_GetTicks() - start < 10 );
+      }
+
       rc = SystemFilter( event );
 
       if ( rc != GUI_NONE ) {
