@@ -164,17 +164,18 @@ void Window::Draw( void ) {
 ////////////////////////////////////////////////////////////////////////
 // NAME       : Window::DrawBack
 // DESCRIPTION: Draw the window background into the internal buffer.
-// PARAMETERS : x - left edge of area to paint
-//              y - top edge of area to paint
-//              w - width of area to paint
-//              h - height of area to paint
+// PARAMETERS : rect - area to paint
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void Window::DrawBack( short x, short y, unsigned short w, unsigned short h ) {
+void Window::DrawBack( const Rect &rect ) {
   const static Image bg( view->GetSystemIcons(), 0, 34, 30, 30 );
-  FillPattern( x, y, w, h, bg, 0, 0 );
-  DrawBox( Rect( 0, 0, Width(), Height() ), BOX_RAISED );
+  Rect clip;
+  FillPattern( rect, bg, 0, 0 );
+  GetClipRect( clip );
+  SetClipRect( rect );
+  DrawBox( Rect(0, 0, w, h), BOX_RAISED );
+  SetClipRect( clip );
 }
 
 ////////////////////////////////////////////////////////////////////////
