@@ -44,7 +44,7 @@ const char *pathdelimiter = "/";
 #endif
 
 const char *mapstrings[5] = {"unknown","BI","BIDD1","BIDD2","HL"};
-const char verstr[] = "0.4.4";
+const char verstr[] = "0.4.9";
 
 const char helpstr[] =
 "Usage: %s mapname [options]\n"
@@ -561,23 +561,23 @@ int main(int argc, char *argv[])
                 i++;
                 break;
             case 't': // map title
-                mapinfo.title = (unsigned char *) malloc( (strlen(argv[i+1])+1) * sizeof(unsigned char) );
+                mapinfo.title = (char *) malloc( (strlen(argv[i+1])+1) * sizeof(char) );
                 strcpy(mapinfo.title, argv[++i]);
                 break;
             case 'i': // map info
-                mapinfo.info = (unsigned char *) malloc( (strlen(argv[i+1])+1) * sizeof(unsigned char) );
+                mapinfo.info = (char *) malloc( (strlen(argv[i+1])+1) * sizeof(char) );
                 strcpy(mapinfo.info, argv[++i]);
                 break;
             case 'o': // output file
-                mapfiles.srcfilename = (unsigned char *) malloc( (strlen(argv[i+1])+1) * sizeof(unsigned char) );
+                mapfiles.srcfilename = (char *) malloc( (strlen(argv[i+1])+1) * sizeof(char) );
                 strcpy(mapfiles.srcfilename, argv[++i]);
                break;
             case 'd': // input dir, when malloc'ing leave space for path delimiter (might be needed and makes things easier)
-                mapfiles.inputdir = (unsigned char *) malloc( (strlen(argv[i+1])+2) * sizeof(unsigned char) );
+                mapfiles.inputdir = (char *) malloc( (strlen(argv[i+1])+2) * sizeof(char) );
                 strcpy(mapfiles.inputdir, argv[++i]);
                 break;
             case 'O': // output dir, when malloc'ing leave space for path delimiter (might be needed and makes things easier)
-                mapfiles.outputdir = (unsigned char *) malloc( (strlen(argv[i+1])+2) * sizeof(unsigned char) );
+                mapfiles.outputdir = (char *) malloc( (strlen(argv[i+1])+2) * sizeof(char) );
                 strcpy(mapfiles.outputdir, argv[++i]);
                 break;
             case 'v': // version information
@@ -596,7 +596,7 @@ int main(int argc, char *argv[])
         }
         else
         {
-            mapinfo.number = (unsigned char *) malloc( (strlen(argv[i])+2) * sizeof(unsigned char) );
+            mapinfo.number = (char *) malloc( (strlen(argv[i])+2) * sizeof(char) );
             strcpy(mapinfo.number,argv[i]);
         }
     }
@@ -614,18 +614,18 @@ int main(int argc, char *argv[])
 
     if( mapfiles.inputdir == NULL )
     {
-        mapfiles.inputdir = (unsigned char *) malloc( 2 * sizeof(unsigned char) );
+        mapfiles.inputdir = (char *) malloc( 2 * sizeof(char) );
         strcpy(mapfiles.inputdir,"");
     }
     else if( mapfiles.inputdir[strlen(mapfiles.inputdir)-1] != pathdelimiter[0] ) strcat(mapfiles.inputdir,pathdelimiter);
 
     i = strlen(mapinfo.number) + strlen(mapfiles.inputdir) + 5;
-    mapfiles.finfilename = (unsigned char *) malloc(i);
+    mapfiles.finfilename = (char *) malloc(i);
     strcpy(mapfiles.finfilename,mapfiles.inputdir);
     strcat(mapfiles.finfilename,mapinfo.number);
     strcat(mapfiles.finfilename,".FIN");
 
-    mapfiles.shpfilename = (unsigned char *) malloc(i);
+    mapfiles.shpfilename = (char *) malloc(i);
     strcpy(mapfiles.shpfilename,mapfiles.inputdir);
     strcat(mapfiles.shpfilename,mapinfo.number);
     strcat(mapfiles.shpfilename,".SHP");
@@ -733,7 +733,7 @@ int main(int argc, char *argv[])
 
     if( mapinfo.title == NULL )
     {
-        mapinfo.title = (unsigned char *) malloc( (strlen(mapinfo.number) + strlen(mapstrings[convdata.type]) + 5) * sizeof(unsigned char) );
+        mapinfo.title = (char *) malloc( (strlen(mapinfo.number) + strlen(mapstrings[convdata.type]) + 5) * sizeof(char) );
         strcpy(mapinfo.title,mapstrings[convdata.type]);
         strcat(mapinfo.title,"-");
         strcat(mapinfo.title,mapinfo.number);
@@ -741,20 +741,20 @@ int main(int argc, char *argv[])
 
     if( mapinfo.info == NULL )
     {
-        mapinfo.info = (unsigned char *) malloc( (strlen(mapinfo.number) + strlen(mapstrings[convdata.type]) + 50) * sizeof(unsigned char) );
+        mapinfo.info = (char *) malloc( (strlen(mapinfo.number) + strlen(mapstrings[convdata.type]) + 50) * sizeof(char) );
         sprintf(mapinfo.info,"This is a conversion of map %s from %s",mapinfo.number,mapstrings[convdata.type]);
     }
 
     if( mapfiles.outputdir == NULL )
     {
-        mapfiles.outputdir = (unsigned char *) malloc( 2 * sizeof(unsigned char) );
+        mapfiles.outputdir = (char *) malloc( 2 * sizeof(char) );
         strcpy(mapfiles.outputdir,"");
     }
     else if( mapfiles.outputdir[strlen(mapfiles.outputdir)-1] != pathdelimiter[0] ) strcat(mapfiles.outputdir,pathdelimiter);
 
     if( mapfiles.srcfilename == NULL )
     {
-        mapfiles.srcfilename = (unsigned char *) malloc( (strlen(mapinfo.title)+strlen(mapfiles.outputdir)+5) * sizeof(unsigned char) );
+        mapfiles.srcfilename = (char *) malloc( (strlen(mapinfo.title)+strlen(mapfiles.outputdir)+5) * sizeof(char) );
         strcpy(mapfiles.srcfilename,mapfiles.outputdir);
         strcat(mapfiles.srcfilename,mapinfo.title);
         strcat(mapfiles.srcfilename,".src");
