@@ -390,7 +390,7 @@ int ShopHandler::ParseSection( ifstream &in, const string *opt, unsigned long &l
         else if ( val == "factory" ) b->SetFlags(BLD_FACTORY);
         else {
           rc = 1;
-          cerr << "Error in line " << line << ": Unknown type '" << val << endl;
+          cerr << "Error in line " << line << ": Unknown type '" << val << "'" << endl;
         }
       } else if ( key == "factory" ) {                            // can produce which units
         const UnitType *type = ParseUnitName( val );
@@ -465,6 +465,7 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
         else if ( val == "unitdestroyed" ) e->SetTrigger(ETRIGGER_UNIT_DESTROYED);
         else if ( val == "unitposition" ) e->SetTrigger(ETRIGGER_UNIT_POSITION);
         else if ( val == "handicap" ) e->SetTrigger(ETRIGGER_HANDICAP);
+        else if ( val == "havecrystals" ) e->SetTrigger(ETRIGGER_HAVE_CRYSTALS);
         else if ( val == "havebuilding" ) {
           e->SetTrigger(ETRIGGER_HAVE_BUILDING);
           e->SetTData(2, -1);
@@ -498,9 +499,14 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
              (key == "tunit") ) e->SetTData(0, StrToNum(val));      // unit
         else if ( (e->Trigger() == ETRIGGER_HAVE_BUILDING) &&
              (key == "tbuilding") ) e->SetTData(0, StrToNum(val));  // building id
+        else if ( (e->Trigger() == ETRIGGER_HAVE_CRYSTALS) &&
+             (key == "tbuilding") ) e->SetTData(2, StrToNum(val));  // building id
+        else if ( (e->Trigger() == ETRIGGER_HAVE_CRYSTALS) &&
+             (key == "tcrystals") ) e->SetTData(0, StrToNum(val));  // number of crystals
         else if ( ((e->Trigger() == ETRIGGER_HAVE_BUILDING) ||
                    (e->Trigger() == ETRIGGER_HAVE_UNIT) ||
-                   (e->Trigger() == ETRIGGER_UNIT_DESTROYED)) &&
+                   (e->Trigger() == ETRIGGER_UNIT_DESTROYED) ||
+                   (e->Trigger() == ETRIGGER_HAVE_CRYSTALS)) &&
              (key == "towner") ) e->SetTData(1, StrToNum(val)-1);   // owner of shop/unit
         else if ( (e->Trigger() == ETRIGGER_UNIT_POSITION) &&
              (key == "towner") ) e->SetTData(2, StrToNum(val)-1);   // owner of unit
@@ -813,6 +819,7 @@ int MapRawHandler::ParseSection( ifstream &in, const string *opt, unsigned long
 
   for ( int i = 0; i < size.y; ++i ) {
     getline(in, buf);
+    ++line;
     if ( in.eof() ) {
       cerr << "Error in line " << line << ": unexpected end of map data" << endl;
       return -1;
@@ -1674,6 +1681,32 @@ int MissionParser::check_events( void ) const {
           return 1;
         }
         break;
+      case ETRIGGER_HAVE_CRYSTALS:
+        if ( e->GetTData(0) == -9999 ) {
+          cerr << "Error: Crystals trigger does not specify amount (" << (short)e->ID() << ")" << endl;
+          return 1;
+        } else if ( ABS(e->GetTData(0)) > 5000 ) {
+          cerr << "Error: Invalid crystals amount " << e->GetTData(0) << " for event trigger ("
+               << (short)e->ID() << ")" << endl;
+          return 1;
+        }
+
+        if ( e->GetTData(1) == -9999 ) e->SetTData(1, e->Player());
+        else if ( (e->GetTData(1) != PLAYER_ONE) && (e->GetTData(1) != PLAYER_TWO) ) {
+          cerr << "Error: Event trigger wants invalid player to own a shop ("
+               << (short)e->ID() << ")" << endl;
+          return 1;
+        }
+
+        if ( e->GetTData(2) < -2 ) e->SetTData(2, -1);
+        else if ( e->GetTData(2) >= 0 ) {
+          if ( !GetBuildingByID( e->GetTData(2) ) ) {
+            cerr << "Error: Event trigger targets non-existing shop " << e->GetTData(2)
+                 << " (" << (short)e->ID() << ')' << endl;
+            return 1;
+          }
+        }
+        break;
       case ETRIGGER_UNIT_POSITION:
         if ( map.Index2Hex( e->GetTData(1) ) == Point(-1,-1) ) {
           cerr << "Error: Invalid location set for event trigger (" << (short)e->ID() << ')' << endl;
